<?php
namespace Controller;

use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class Acl extends Simple {

	/** @var \Service\Appropriation $appropriationService  */
	protected $aclService;

	public function __construct(\Service\Acl $aclService) {
		$this->aclService = $aclService;
	}

	public function findElementsByProfile(Request\Filter $request, Response $response) {
		$param = $request->getFilterCriteria()->getConditions();
		$profile = empty($param[0]['value']) || $param[0]['value'] === 'undefined' || $param[0]['value'] === 'null' ? 0 : $param[0]['value'];

		$dataSet = array();

		try{
			$dataSet = $this->aclService->getElementsByProfile($profile);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus elementos, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('element', $dataSet));
	}

	public function findProfiles(Request\Filter $request, Response $response) {
		$param = $request->getFilterCriteria()->getConditions();
        $profile = Utilities::getValueInFilterDataSourceByName($param,'profile');
		$dataSet = array();

		try{
			$dataSet = $this->aclService->getProfiles($profile);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus grupos de usuários, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('profile', $dataSet));
	}

	public function saveProfile(Request\DataSet $request, Response $response) {

        $profiles = $request->getDataSet()->getRows();

		try{
		 	foreach ($profiles as $profile) {
                $this->persistProfile($profile);
		 	 }
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível salvar os perfis de usuário, pois " . $ex->getMessage()));
		}
	}

	public function saveRule(Request\DataSet $request, Response $response) {

        $rules = $request->getDataSet()->getRows();

		try{
		 	foreach ($rules as $rule) {
		 		$ruleId = empty($rule['RULE_ID']) ? null : $rule['RULE_ID'];
		 		$element = $rule['ID'];
		 		$profile = $rule['PROFILE'];
		 		$permission = $rule['RULE'];
				$this->aclService->persistRule($ruleId, $element, $profile, $permission);
		 	 }
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível salvar as regras, pois " . $ex->getMessage()));
		}
	}

	public function deleteProfile(Request\DataSet $request, Response $response){
		$count = 0;
		$dataSet = $request->getDataSet()->getRows();

		foreach ($dataSet as $param) {
			try{
                $this->removeProfile($param);
            }catch(\Exception $ex){
				$response->addMessage(new Response\Message("Não foi possível deletar o perfil de acesso, pois " . $ex->getMessage()));
				$count--;
			}
			$count++;
		}
		$response->addMessage(new Response\Message("$count perfis de acesso deletados."));
	}

	public function findResourcesByProfile(Request\Filter $request, Response $response) {
		$param = $request->getFilterCriteria()->getConditions();
		$profile = empty($param[0]['value']) || $param[0]['value'] === 'undefined' || $param[0]['value'] === 'null' ? 0 : $param[0]['value'];

		$dataSet = array();

		try{
			$dataSet = $this->aclService->getResourcesByProfile($profile);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus recursos associados a um perfil, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('element', $dataSet));
	}

	public function saveProfileResources(Request\DataSet $request, Response $response) {

        $profileResources = $request->getDataSet()->getRows();

		try{
		 	foreach ($profileResources as $profileResource) {
                $this->persistProfileResource($profileResource);
            }
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível salvar os recursos do perfil, pois " . $ex->getMessage()));
		}
	}

	public function removeProfileResources(Request\DataSet $request, Response $response) {

        $profileResources = $request->getDataSet()->getRows();

		try{
		 	foreach ($profileResources as $profileResource) {
                $this->deleteProfileResource($profileResource);
            }
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível salvar os recursos do perfil, pois " . $ex->getMessage()));
		}
	}

    /**
     * @param $profile
     * @return null|object|\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile
     */
    public function persistProfile($profile)
    {
        $id = empty($profile['ID']) ? null : $profile['ID'];
        $structure = empty($profile['STRUCTURE']) ? 0 : $profile['STRUCTURE'];
        $category = empty($profile['CATEGORY_CONTROL']) ? 0 : $profile['CATEGORY_CONTROL'];
        $control = empty($profile['CONTROL']) ? null : $profile['CONTROL'];
        return $this->aclService->persistProfile($id, $profile['NAME'], $category, $structure, $control);
    }

    /**
     * @param $param
     * @throws \Exception
     */
    protected function removeProfile($param)
    {
        $this->aclService->deleteProfile($param['ID']);
    }

    /**
     * @param $profileResource
     * @throws \Exception
     */
    protected function deleteProfileResource($profileResource)
    {
        $this->aclService->deleteProfileResource($profileResource['ID'], $profileResource['PROFILE']);
    }

    /**
     * @param $profileResource
     * @throws \Exception
     */
    protected function persistProfileResource($profileResource)
    {
        $this->aclService->persistProfileResource($profileResource['ID'], $profileResource['PROFILE']);
    }
}