<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 18/08/14
 * Time: 14:07
 */

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\PmCache;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\TaskCalendarInterface;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Util\Utilities;

class Calendar
{

    const CALENDAR_DEFAULT = 'D';
    const CALENDAR_STRUCTURE = 'S';
    const CALENDAR_PROJECT = 'P';
    const CALENDAR_RESOURCE = 'R';
    const CALENDAR_ALLOCATION = 'A';

    const DEFAULT_HOUR_WORK_TYPE = '8H';
    const DEFAULT_HOUR_OUT_DAY_TYPE = '0';

    const DEFAULT_NAME_WORK_WEEK = 'Padrão';

    /**
     * @var Utilities
     */
    protected $environment;
    /**
     * @var Repositories\Calendar
     */
    public  $repositoryCalendar;
    /**
     * @var Repositories\ResourceProject
     */
    public  $repositoryResourceProject;
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var \Factory\Calendar
     */
    private $calendarFactory;
    /**
     * @var CalendarWeeklyDays
     */
    private $calendarWeeklyDaysService;
    /**
     * @var CalendarDates
     */
    private $calendarDatesService;
    /**
     * @var CalendarDetail
     */
    private $calendarDetailService;

    public function __construct(EntityManager $entityManager, Utilities $environment,
                                CalendarWeeklyDays $calendarWeeklyDaysService,CalendarDates $calendarDatesService,
                                \Factory\Calendar $calendarFactory,CalendarDetail $calendarDetailService)
    {

        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->calendarFactory = $calendarFactory;
        $this->calendarWeeklyDaysService = $calendarWeeklyDaysService;
        $this->calendarDatesService = $calendarDatesService;
        $this->repositoryCalendar = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR . 'Calendar');
        $this->repositoryResourceProject = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR . 'ResourceProject');
        $this->calendarDetailService = $calendarDetailService;
    }

    public function saveProjectCalendar($projectId, $calendarId, $hourDay, $monthlyDays, $name, $weeklyHours, $days, $type = 'P')
    {

        $project = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project', $projectId);
        if (empty($hourDay) || $hourDay == 0) {
            $hourDay = \Service\Calendar\TaskCalendar::HOUR_DAY;
        }
        if ($calendarId) {
            $calendar = $this->repositoryCalendar->find($calendarId);
            $calendar = $this->updateCalendar($calendar, $hourDay, $monthlyDays, $name, $weeklyHours, $days, $type);
        } else {
            $calendar = $this->createCalendar($hourDay, $monthlyDays, $name, $weeklyHours, $days, $type);
            $project->setCalendar($calendar);
            $this->entityManager->persist($calendar);
            $this->entityManager->flush();
        }
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task[] $tasks */
        $tasks = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Task')->findBy(array('project' => $project, 'isIssue' => 'N'));
        if ($tasks && is_array($tasks)) {
            foreach ($tasks as $task) {
                $task->setEstimatedBeginDate(clone $task->getEstimatedBeginDate());
                $this->entityManager->persist($task);
            }
            $this->entityManager->flush();
        } else {
            $project->setEstimatedBeginDate(clone $project->getEstimatedBeginDate());
            $this->entityManager->persist($project);
            $this->entityManager->flush();
        }

        return $calendar;
    }
    public function createCalendar($hourDay,$monthlyDays,$name,$weeklyHours,$days,$type){

        $entity = $this->calendarFactory->createCalendarOdd($hourDay,$monthlyDays,$name,$weeklyHours,$days,$type);
        $this->entityManager->persist($entity);
        $this->entityManager->flush();

        return $entity;
    }
    public function updateCalendar($calendar ,$hourDay,$monthlyDays,$name,$weeklyHours,$days,$type){

        $entity = $this->calendarFactory->updateCalendarOdd($calendar, $hourDay,$monthlyDays,$name,$weeklyHours,$days,$type);
        $this->entityManager->persist($entity);
        $this->entityManager->flush();

        return $entity;
    }

    /**
     * @param $calendarId
     * @return Entities\Calendar
     */
    public function find($calendarId){

        $calendar = $this->repositoryCalendar->find($calendarId);

        return $calendar;
    }

    public function copyAllCalendarDetail(Entities\Calendar $originCalendar,Entities\Calendar $destinationCalendar){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail[] $calendarDetails */
        $calendarDetails = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR . 'CalendarDetail')
            ->findBy(array('calendar'=>$originCalendar->getId()),array('id'=>'asc'));
        foreach ($calendarDetails as $calendarDetail){
            $calendarDetailDuplicate = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR . 'CalendarDetail')
                ->findoneBy(array('calendar'=>$destinationCalendar->getId(),'originCalendarDetail' =>$calendarDetail->getId()));
            if(!$calendarDetailDuplicate){
                $calendarDetailClone = clone $calendarDetail;
                $calendarDetailClone->setCalendar($destinationCalendar);
                if(!$calendarDetailClone->getOriginCalendarDetail()){
                    $calendarDetailClone->setOriginCalendarDetail($calendarDetail);
                }
                $this->entityManager->persist($calendarDetailClone);
                $this->calendarWeeklyDaysService->copyAllCalendarWeeklyDaysService($calendarDetail,$calendarDetailClone);
            }
        }

    }
    public function saveException($id,$recurrenceType,$name,$calendar,$type,$status,$beginDate,$occurrences,
                                  $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
                                  $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime){
        $calendarDetail = $this->calendarDetailService->saveException($id,$recurrenceType,$name,$calendar,$type,$status,$beginDate,$occurrences,
            $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
            $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime);
        return $calendarDetail;

    }
    public function saveEditCalendar($id, $name, $monthlyDays, $weeklyHours, $status, $type)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($id) {
                $calendar = $this->repositoryCalendar->find($id);
            } else {
                $calendar = new Entities\Calendar();
            }
            
            $calendar = $this->calendarFactory->updateCalendar($calendar, $name, $monthlyDays, $weeklyHours, $status, $type);
            $this->entityManager->persist($calendar);
            if (empty($id) && $type == self::CALENDAR_DEFAULT) {
                $this->calendarWeeklyDaysService->saveWeeklyDays($id, $calendar, self::DEFAULT_NAME_WORK_WEEK,
                    CalendarDetail::OUT_DAY_TYPE, self::DEFAULT_HOUR_OUT_DAY_TYPE, CalendarDetail::WORK_TYPE,
                    self::DEFAULT_HOUR_WORK_TYPE, CalendarDetail::WORK_TYPE, self::DEFAULT_HOUR_WORK_TYPE,
                    CalendarDetail::WORK_TYPE, self::DEFAULT_HOUR_WORK_TYPE, CalendarDetail::WORK_TYPE,
                    self::DEFAULT_HOUR_WORK_TYPE, CalendarDetail::WORK_TYPE, self::DEFAULT_HOUR_WORK_TYPE,
                    CalendarDetail::OUT_DAY_TYPE, self::DEFAULT_HOUR_OUT_DAY_TYPE, null, null);
            }
            $this->entityManager->flush($calendar);
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $calendar;
    }
    public function findExceptionCalendarDates($calendarId,$month,$year)
    {
        list($beginDate, $endDate) = Utilities::getBeginAndEndDateByMonth($month,$year);

       return $this->findExceptionCalendarDatesWithCalendarDefault($calendarId,$beginDate,$endDate);
    }

    /**
     * @param $calendarId
     * @param $beginDate
     * @param $endDate
     * @param bool $inSeconds
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     * @throws \Doctrine\ORM\TransactionRequiredException
     * @return array
     */
    public function findExceptionCalendarDatesWithCalendarDefault($calendarId,$beginDate,$endDate,$inSeconds = false)
    {

        /** @var Entities\Calendar $calendar */
        $calendar = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Calendar', $calendarId);
        if($calendar && $calendar->getCalendarDefault()){
            $dates = $this->calendarDatesService->findExceptionCalendarDatesByCalendarDefault(
                $calendarId,$calendar->getCalendarDefault()->getId(),$beginDate,$endDate,$inSeconds);
        }else{
            $dates = $this->calendarDatesService->findExceptionCalendarDates($calendarId,$beginDate,$endDate,$inSeconds);

        }
        if(!$inSeconds){
            foreach ($dates as &$date){
                $date['DAILY_HOURS'] = Utilities::secondsToTime($date['DAILY_HOURS']);
            }
        }
        if($calendar && $calendar->getCalendarDefault()){
            $defaultDates = $this->calendarDatesService->findExceptionCalendarDates(
                $calendar->getCalendarDefault()->getId(),$beginDate,$endDate);
            if(!$inSeconds){
                foreach ($defaultDates as &$defaultDate){
                    $defaultDate['DAILY_HOURS'] = Utilities::secondsToTime($defaultDate['DAILY_HOURS']);
                }
            }
            $dates = $this->mergeCalendarDates($dates,$defaultDates);
        }
        return $dates;
    }

    public function findExceptionCalendarDatesStructure($structureId,$month,$year)
    {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure $structure */
        $structure = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Structure')->find($structureId);
        $calendar = Utilities::getNextCalendar($structure);
        if($calendar){
            $calendarId = $calendar->getId();
        }else{
            $calendarId = 0;
        }
        return $this->findExceptionCalendarDates($calendarId,$month,$year);
    }

    public function findCalendarExceptionByResource(Entities\Resource $resource, Entities\Structure $structure,$id = null)
    {
        return $this->calendarDetailService->findCalendarExceptionByResource($resource,$structure,$id);
    }

    public function findCalendarWeeklyDaysByResource(Entities\Resource $resource, Entities\Structure $structure,$id = null)
    {
        return $this->calendarWeeklyDaysService->findCalendarWeeklyDaysByResource($resource,$structure,$id);
    }

    public function findCalendarExceptionByProject(Entities\Project $project, Entities\Structure $structure,$id = null, $filter = '')
    {
        return $this->calendarDetailService->findCalendarExceptionByProject($project,$structure,$id, $filter);
    }

    public function findCalendarExceptionByProjectResource(Entities\ResourceProject $projectResource,$id = null)
    {
        return $this->calendarDetailService->findCalendarExceptionByProjectResource($projectResource,$id);
    }

    public function findCalendarWeeklyDaysByProject(Entities\Project $project, Entities\Structure $structure,$id = null, $filter = '')
    {
        return $this->calendarWeeklyDaysService->findCalendarWeeklyDaysByProject($project,$structure, $id, $filter);
    }

    public function findCalendarWeeklyDaysByProjectResource(Entities\ResourceProject $projectResource, $id = null)
    {
        return $this->calendarWeeklyDaysService->findCalendarWeeklyDaysByProjectResource($projectResource, $id);
    }

    public function findExceptionCalendarDatesResource(Entities\Resource $resource,$structure,$beginDate, $endDate,
                                                       $inSeconds = false)
    {
        if($structure){
            $structureId = $structure->getId();
        }else{
            $structureId = 0;
        }
        $key = PmCache::getInstance()->keyfy(array("findExceptionCalendarDatesResource", $resource->getId(), $structureId,
            $beginDate, $endDate));
        if(!PmCache::getInstance()->has($key)) {
            $resourceCalendarDates = array();
            $structureCalendarDates = array();
            if ($resource->getCalendar()) {
                $resourceCalendarDates = $this->findExceptionCalendarDatesWithCalendarDefault($resource->getCalendar()->getId(),
                    $beginDate, $endDate, $inSeconds);
            }
            if ($structure) {
                $calendar = Utilities::getNextCalendar($structure);
                if ($calendar) {
                    $calendarId = $calendar->getId();
                } else {
                    $calendarId = 0;
                }
                $structureCalendarDates = $this->findExceptionCalendarDatesWithCalendarDefault($calendarId, $beginDate,
                    $endDate, $inSeconds);
            }
            PmCache::getInstance()->set($key, $this->mergeCalendarDates($resourceCalendarDates, $structureCalendarDates));
        }
        return  PmCache::getInstance()->get($key);
    }

    public function findExceptionCalendarDatesProject(Entities\Project $project,$structure,$beginDate, $endDate,$inSeconds = false)
    {
        $key = PmCache::getInstance()->keyfy(array("findExceptionCalendarDatesProject", $project->getId(), $beginDate, $endDate));
        if(!PmCache::getInstance()->has($key)) {

            $resourceCalendarDates = array();
            $structureCalendarDates = array();
            if ($project->getCalendar()) {
                $resourceCalendarDates = $this->findExceptionCalendarDatesWithCalendarDefault($project->getCalendar()->getId(),
                    $beginDate, $endDate, $inSeconds);
            }
            if ($structure) {
                $calendar = Utilities::getNextCalendar($structure);
                if ($calendar) {
                    $calendarId = $calendar->getId();
                } else {
                    $calendarId = 0;
                }
                $structureCalendarDates = $this->findExceptionCalendarDatesWithCalendarDefault($calendarId, $beginDate,
                    $endDate, $inSeconds);
            }
            PmCache::getInstance()->set($key, $this->mergeCalendarDates($resourceCalendarDates, $structureCalendarDates));
        }
        return  PmCache::getInstance()->get($key);

    }

    public function findExceptionCalendarDatesTask(Entities\Task $task,$beginDate, $endDate,$inSeconds = false)
    {
        $key = PmCache::getInstance()->keyfy(array("findExceptionCalendarDatesTask", $task->getId(), $beginDate, $endDate));
        if(!PmCache::getInstance()->has($key)) {
            $allocations = $task->getAllocations();
            $project = $task->getProject();
            if (count($allocations) > 0) {
                $resourceCalendarDates = array();
                foreach ($allocations as $allocation) {
                    $resource = $allocation->getResource();
                    /** @var Entities\ResourceProject $resourceProject */
                    $resourceProject = $this->repositoryResourceProject
                        ->findOneBy(array('resourceC' => $resource, 'project' => $project));
                    if ($resourceProject) {
                        $resourceCalendarDates[] = $this->findExceptionCalendarDatesProjectResource($resourceProject,
                            $beginDate, $endDate, $inSeconds);
                    }
                }
                if($resourceProject){
                    $calendarDates = $this->mergeCalendarDatesTaskResource($resourceCalendarDates);
                }else{
                    $structure = $project->getStructure();
                    $calendarDates = $this->findExceptionCalendarDatesProject($project, $structure, $beginDate, $endDate,
                        $inSeconds);
                }
            } else {
                $structure = $project->getStructure();
                $calendarDates = $this->findExceptionCalendarDatesProject($project, $structure, $beginDate, $endDate,
                    $inSeconds);
            }
            PmCache::getInstance()->set($key, $calendarDates);
        }
        return PmCache::getInstance()->get($key);
    }

    /**
     * @param Entities\ResourceProject $resourceProject
     * @param $beginDate
     * @param $endDate
     * @param bool $inSeconds
     * @return array
     */
    public function findExceptionCalendarDatesProjectResource(Entities\ResourceProject $resourceProject,$beginDate, $endDate, $inSeconds = false)
    {
        $key = PmCache::getInstance()->keyfy(array("findExceptionCalendarDatesProjectResource", $resourceProject->getId(),$beginDate, $endDate));
        if(!PmCache::getInstance()->has($key)) {
            if ($resourceProject->getCalendar()) {
                $resourceCalendarDates = $this->findExceptionCalendarDatesWithCalendarDefault($resourceProject->getCalendar()
                    ->getId(), $beginDate, $endDate, $inSeconds);
            } else {
                $resource = $resourceProject->getResourceC();
                $structure = $this->environment->getStructureForResource($resource);
                $resourceCalendarDates = $this->findExceptionCalendarDatesResource($resource, $structure, $beginDate, $endDate, $inSeconds);
            }
            $project = $resourceProject->getProject();
            $structure = $project->getStructure();
            $projectCalendarDates = $this->findExceptionCalendarDatesProject($project, $structure, $beginDate, $endDate, $inSeconds);

            PmCache::getInstance()->set($key, $this->mergeCalendarDates($projectCalendarDates,$resourceCalendarDates));
        }
        return PmCache::getInstance()->get($key);


    }

    public function defaultCalendarDelete($id)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $calendar = $this->repositoryCalendar->find($id);
            $calendarsDetail = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR . 'CalendarDetail')
                ->findby(array('calendar' => $calendar));
            foreach ($calendarsDetail as $calendarDetail) {
                $this->calendarWeeklyDaysService->removeAllWeeklyDays($calendarDetail,true);
                $this->entityManager->remove($calendarDetail);
            }
            $this->entityManager->flush();
            $this->entityManager->remove($calendar);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    public function calendarDelete($id)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $calendar = $this->repositoryCalendar->find($id);
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail[] $calendarDetails */
            $calendarDetails = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR . 'CalendarDetail')
                ->findby(array('calendar' => $calendar));
            foreach ($calendarDetails as $calendarDetail) {
                $this->calendarWeeklyDaysService->removeAllWeeklyDays($calendarDetail,true);
                $originCalendarDetails = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR . 'CalendarDetail')
                    ->findby(array('originCalendarDetail' => $calendarDetail));
                if(!empty($originCalendarDetails)){
                    foreach($originCalendarDetails as $originCalendarDetail){
                        $this->calendarWeeklyDaysService->removeAllWeeklyDays($originCalendarDetail,true);
                        $this->entityManager->remove($originCalendarDetail);
                    }
                }
                $this->entityManager->remove($calendarDetail);
            }
            $this->entityManager->flush();
            $this->entityManager->remove($calendar);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    public function saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
                                   $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                                   $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate){
        return $this->calendarWeeklyDaysService->saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
            $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
            $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
    }
    public function getDataSetForEditException(){
        return array();
    }
    public function removeAllCalendarDetailWithOriginCalendarDetail(Entities\Calendar $calendar){
        /** @var Entities\CalendarDetail[] $calendarDetails */
        $calendarDetails = $this->calendarDetailService->calendarDetailRepository->findBy(array('calendar' =>$calendar));
        foreach($calendarDetails as $calendarDetail){
            if($calendarDetail->getOriginCalendarDetail()){
                $this->calendarWeeklyDaysService->removeAllWeeklyDays($calendarDetail,true);
                $this->entityManager->remove($calendarDetail);
                $this->entityManager->flush();
            }
        }
        return array();
    }
    public function adjustCalendarToNewParent(Entities\Calendar $calendar,Entities\Calendar $calendarParent){
        $this->removeAllCalendarDetailWithOriginCalendarDetail($calendar);
        $this->copyAllCalendarDetail($calendarParent,$calendar);
        $this->entityManager->flush();
        return $calendar;
    }
    public function getCalendarDates(Entities\Calendar $calendar, $beginDate, $endDate){
        return $this->calendarDatesService->findExceptionCalendarDatesByCalendarDefault($calendar->getId(),
            $calendar->getCalendarDefault()->getId(),$beginDate,$endDate);
    }

    /**
     * regra de exception
     * 1 - $primaryDates[IS_EXCEPTION] = Y
     * 2 - $secondaryDates[IS_EXCEPTION]  = Y
     * 3 - $primaryDates[IS_EXCEPTION] = N
     * 4 - $secondaryDates[IS_EXCEPTION] = N
     * 5 - $primaryDates[IS_EXCEPTION] = null
     * 6 - $secondaryDates[IS_EXCEPTION] = null
     * @param array $primaryDates
     * @param array $secondaryDates
     * @return array
     */
    public function mergeCalendarDates(array $primaryDates,array $secondaryDates){
        $calendarDates = array();
        if(empty($secondaryDates)){
            $calendarDates = $primaryDates;
        }else if(empty($primaryDates)){
            $calendarDates = $secondaryDates;
        }else{
            foreach($primaryDates as $key => $date ){
                if($date['IS_EXCEPTION'] == 'Y'){
                    $calendarDates[] = $date;
                }else if($secondaryDates[$key]['IS_EXCEPTION'] == 'Y'){
                    $calendarDates[] = $secondaryDates[$key];
                }else if($date['IS_EXCEPTION'] == 'N'){
                    $calendarDates[] = $date;
                }else if($secondaryDates[$key]['IS_EXCEPTION'] == 'N'){
                    $calendarDates[] = $secondaryDates[$key];
                }else if($date['IS_EXCEPTION']){
                    $calendarDates[] = $date;
                }else if($secondaryDates[$key]['DAILY_HOURS'] >= $date['DAILY_HOURS']){
                    $calendarDates[] = $date;
                }else {
                    $calendarDates[] = $secondaryDates[$key];
                }
            }
        }
        return $calendarDates;
    }

    public function mergeCalendarDatesTaskResource(array $resourcesCalendarDates){
        $calendarDates = null;
        foreach($resourcesCalendarDates as $resourceCalendarDates){
            if(!$calendarDates){
                $calendarDates = $resourceCalendarDates;
            }else{
                foreach($resourceCalendarDates as $key => $date ){
                    if($date['DAILY_HOURS'] < $calendarDates[$key]['DAILY_HOURS']){
                        $calendarDates[$key] = $date;
                    }
                }
            }

        }
        return $calendarDates;
    }

    public function findCalendarDatesForCalculate($entity,$beginDate,$endDate){
        //TODO
        if(is_a($entity, Utilities::ENTITY_DIR . 'Project')){
            /** @var Entities\Project $entity */
            $dateCalendar = $this->findExceptionCalendarDatesProject($entity,$entity->getStructure(),$beginDate,$endDate,true);
        }else{
            /** @var Entities\Task $entity */
            $dateCalendar = $this->findExceptionCalendarDatesTask($entity,$beginDate,$endDate,true);
        }
        return $dateCalendar;
    }

    public function getPreviousTimeInNotTaskChildren(TaskCalendarInterface $task,TaskCalendarInterface $taskFist){
        $previousTime = 0;
        /** @var Entities\TaskRelationship $precursorTaskRelationship */
        $precursorTaskRelationship = InstanceProvider::getRepository()->getTaskRelationship()
            ->findOneBy(array('task' => $task, 'typePrecursor' => Entities\TaskRelationship::END_START_TYPE, 'type' => Entities\TaskRelationship::PRECURSOR_TYPE));
        if ($precursorTaskRelationship) {
            $precursorTask = $precursorTaskRelationship->getTaskRelationship();
            if (($precursorTask->getParent() && $precursorTask->getParent()->getId() != $taskFist->getParent()->getId())
                || is_null($precursorTask->getParent())) {
                if ($taskFist->getEstimatedBeginDate()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE) ==
                    $precursorTask->getEstimatedEndDate()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE)
                ) {
                    $previousTime = $precursorTask->getDuration() + $precursorTask->getPreviousTime();

                }
            } else {
                $previousTime = $this->getPreviousTimeInNotTaskChildren($precursorTask, $taskFist);
            }
        }
        return $previousTime;
    }

    public function getLastTaskChildren(TaskCalendarInterface $task)
    {
        $taskRepository = InstanceProvider::getRepository()->getTask();
        $lastTime = 0;
        /** @var Entities\Task $lastTask */
        $lastTaskId = InstanceProvider::getRepository()->getTask()->getlastTaskId($task->getId());
        if ($lastTaskId) {
            $lastTask = $taskRepository->find($lastTaskId);
            $beginDate = $lastTask->getEstimatedBeginDate()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
            $endDate = $lastTask->getEstimatedEndDate()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
            $calendarDates = $this->findCalendarDatesForCalculate($lastTask, $beginDate, $endDate);
            $previousTimePrecursor = $this->getPreviousTimeInNotTaskChildren($lastTask,$lastTask);
            $duration = $lastTask->getDuration() + $lastTask->getPreviousTime();
            foreach ($calendarDates as $calendarDate) {
                if ($calendarDate['TYPE'] == CalendarDetail::WORK_TYPE) {
                    $dayHours = intval($calendarDate['DAILY_HOURS']);
                    if ($dayHours > $duration) {
                        $lastTime = $duration;
                    } else {
                        $duration -= $dayHours;
                    }
                }

            }
            $lastTime = $lastTime - $previousTimePrecursor;
        }
        return $lastTime ;
    }

    public function getDayHourByDate($date,$calendarDates){
        foreach($calendarDates as $calendarDate){
            if($calendarDate['DATA'] == $date ){
                if( $calendarDate['TYPE'] == \Service\CalendarDetail::OUT_DAY_TYPE){
                    return 0;
                }else{
                    return intval($calendarDate['DAILY_HOURS']);
                }
            }
        }
        throw new \Exception('date not found in CalendarDates');
        return false;
    }


}