<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities as EntitiesUtil;

class Project extends EntityRepository {

	/**
	 * @param $projectId
	 * @return array
     */
	public function getChildren($projectId){
        $projectIds = Utilities::getAllProjectChildrenIdInArray($projectId);
        $projects = $this->findBy(array('id' => $projectIds));
        $projectIds[] = $projectId;
        /** @var Task $repositoryTask */
        $repositoryTask = $this->getEntityManager()
            ->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Task');

        $tasks = $repositoryTask->findBy(array('id' => Utilities::getAllTaskInProjectIdInArray($projectId)));
        return array_merge($tasks, $projects);
	}

    public function getProjectChildrenByIdForCount($projectId){
        $projectIds = Utilities::getAllProjectChildrenIdInArray($projectId);
        $projectIdFilter = "AND (" . EntitiesUtil::fixBugWhereINQuery($projectIds, 'PRO.ID') . ")";
        $projects = $this->getAllChildrenByProjectIds($projectIdFilter);
        return $projects;
	}
	
	
	public function getProjectChildrenById($projectId){
        $projectIds = Utilities::getAllProjectChildrenIdInArray($projectId);
        $projects = $this->findBy(array('id' => $projectIds));
        return $projects;
	}
	
	

    public function getProjectChildrenNotCanceledAndNotConcludedById($projectId){
        $projectIds = Utilities::getAllProjectChildrenNotCanceledAndNotConcludedIdInArray($projectId);
        $projects = $this->findBy(array('id' => $projectIds));
        return $projects;
	}

    /**
     * @param $projectId
     * @return Entities\Project[]
     */
    public function getAllChildrenByProjectIds($filter){
        $query = <<<SQL
	  SELECT PRO.ID
        FROM PROJECT PRO
        
        WHERE PRO.STATUS = :isActive
        $filter
SQL;
        $params = array(
            'isActive' => Entities\Project::IS_ACTIVE
        );
        	$connection = $this->getEntityManager()->getConnection();
	    	return $connection->fetchAll($query, $params);
	}
	
	/**
     * @param $projectId
     * @return Entities\Project[]
     */
    public function getAllProjectRelationshipRestricted($projectId){
        $query = <<<SQL
	  SELECT PRO.ID
        FROM PROJECT PRO
        JOIN PROJECT_RELATIONSHIP PRE ON (PRE.PROJECT_RELATIONSHIP IN (  SELECT P.ID
                                                                           FROM PROJECT P
                                                                          WHERE P.ID IN (:projectId)
                                                                            AND P.STATUS = :isActive
                                                                          UNION ALL
                                                                         SELECT P.ID
                                                                           FROM PROJECT P
                                                                          WHERE P.STATUS = :isActive
                                                                          START WITH P.PARENT IN (:projectId)
                                                                        CONNECT BY PRIOR P.ID = P.PARENT)
                                                                  AND PRE.PROJECT = PRO.ID AND PRE.TYPE = :modelLink )
        JOIN TASK_STATUS TST ON (TST.ID = PRO.PROJECT_STATUS AND TST.CONTROL <> :taskStatusConcluded)
        JOIN PROJECT_RELATIONSHIP PRG ON(PRG.PROJECT = PRO.ID AND PRG.TYPE = :modelLinkGeneration )
        JOIN PROJECT POG ON(POG.ID = PRG.PROJECT_RELATIONSHIP )
        JOIN PROJECT_TYPE PRT ON (PRT.ID = POG.PROJECT_TYPE AND PRT.EXTERNALTYPE = :modelRestrictedControl)
        WHERE PRO.STATUS = :isActive
        AND POG.STATUS = :isActive
SQL;
        $params = array(
            'modelRestrictedControl' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_RESTRICTED,
            'projectId' => $projectId,
            'modelLinkGeneration' => Entities\ProjectRelationship::TYPE_MODEL_GENERATION,
            'modelLink' => Entities\ProjectRelationship::TYPE_MODEL,
            'isActive' => Entities\Project::IS_ACTIVE,
            'taskStatusConcluded' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
        );
        $connection = $this->getEntityManager()->getConnection();
        $projectIds = Utilities::getArrayFromArrayAssociative($connection->fetchAll($query, $params));
        $projects = $this->findBy(array('id' => $projectIds));
        return $projects;
	}

	/**
	 * @param $treeKey
	 * @return array
     */
	public function getProjectChildren($treeKey){

	    $taskStatusConcludedId = $this->getTaskStatusConcludedId();
	    $taskStatusCanceledId = $this->getTaskStatusCanceledId();

		$query = $this->getEntityManager()
			  ->createQueryBuilder()
			  ->select('u')
			  ->from('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project','u')
			  ->andWhere('u.treeKey like :parentTreeKey')
			  ->setParameter("parentTreeKey",$treeKey.".%")
			  ->andWhere('u.status <> :dele')
			  ->setParameter("dele",'D')
			  ->andWhere('u.projectStatus <> :concluido')
			  ->setParameter("concluido", $taskStatusConcludedId)
			  ->andWhere('u.projectStatus <> :cancelado')
			  ->setParameter("cancelado", $taskStatusCanceledId);
		$projects = $query->getQuery()->getResult();
		return $projects;
	}

	public function getTaskStatusConcludedId()
	{
        $taskStatusConcluded = InstanceProvider::getRepository()
                                                   ->getTaskStatus()
                                                   ->findOneBy(array(
                                                        'control'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
                                                        'isProject' => Entities\TaskStatus::IS_PROJECT_YES
                                                    ));
        return $taskStatusConcluded->getId();
	}

	public function getTaskStatusCanceledId()
	{
        $taskStatusCanceled = InstanceProvider::getRepository()
                                                   ->getTaskStatus()
                                                   ->findOneBy(array(
                                                        'control'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
                                                        'isProject' => Entities\TaskStatus::IS_PROJECT_YES
                                                    ));
        return $taskStatusCanceled->getId();
	}

	public function getProjectChildrenNotConcludedOrCanceled($projectId, $filter = ''){

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $query = <<<SQL
       SELECT P.ID
               FROM PROJECT P
               LEFT JOIN TASK_STATUS PRS ON(PRS.ID = P.PROJECT_STATUS)
               LEFT JOIN PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
              WHERE P.STATUS <> 'D'
              $filterTypeProject
              $filter
                AND PRS.CONTROL NOT IN (:controlCanceled, :controlConcluded, :controlHomologated)
              START WITH P.ID = $projectId
            CONNECT BY PRIOR P.ID = P.PARENT
SQL;
        $params = array(
            'controlCanceled'    => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'controlConcluded'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'controlHomologated' =>  Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED,
        );
        return Utilities::arrayAssociativeToArray(self::getEntityManager()->getConnection()->fetchAll($query, $params));
	}


	/**
	 * @param $organizationId
	 * @param $managerId
	 * @param null $projectExclusionId
     * @param array(0) $projectExclusionStatus
	 * @return array
     */
	public function findProjectsByManager($organizationId, $managerId, $projectExclusionId = null, $projectExclusionStatus = array()) {
        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($managerId);
        $allProjectsExclusionStatus = implode($projectExclusionStatus, ',');
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
		$query = <<<SQL
      SELECT P.ID, P.NAME, P.PARENT,PT.LABEL AS PROJECT_TYPE_NAME
        FROM PROJECT P
        JOIN STRUCTURE ST ON(ST.ID = P.STRUCTURE)
   LEFT JOIN PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
       WHERE P.ORGANIZATION = :ORGANIZATION
             $filterTypeProject
         AND P.STATUS = 'A'
         AND $stringProjectByManager
         AND P.PROJECT_STATUS NOT IN (SELECT TS.ID FROM TASK_STATUS TS WHERE TS.CONTROL IN (:EXCLUDED_STATUS, :CANCELED_STATUS))
         AND P.ID <> :PROJECT_ID
    ORDER BY P.NAME
SQL;
		$params = array(
			'ORGANIZATION' => $organizationId,
			'MANAGER' => $managerId,
			'PROJECT_ID' => $projectExclusionId,
            'EXCLUDED_STATUS' => $allProjectsExclusionStatus,
            'CANCELED_STATUS' => Entities\Project::PROJECT_CANCELED_STATUS
		);
		$connection = $this->getEntityManager()->getConnection();
		return $connection->fetchAll($query, $params);
	}

    public function getAppropriationByProject($projectId) {
        $query = <<<SQL
        SELECT A.ID, A.DESCRIPTION
        FROM APPROPRIATION A
        JOIN TASK T ON T.ID = A.TASK
        JOIN PROJECT P ON P.ID = T.PROJECT
        WHERE P.ID = :project

SQL;
        $params = array(
            'project' => $projectId
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }


    public function findAllProjectsByModel($projectId) {
        $query = <<<SQL
          SELECT PJ.ID, PJ.NAME FROM PROJECT PJ
            JOIN TASK_STATUS TS ON PJ.PROJECT_STATUS = TS.ID
           WHERE TS.CONTROL NOT IN ('C', 'X')
             AND PJ.STATUS = 'A'
           START WITH PJ.ID in (:PROJECT_ID)
         CONNECT BY PRIOR PJ.ID = PJ.PARENT
SQL;

        $params = array( 'PROJECT_ID' => $projectId );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }

    public function findTasksNamesFromProject($projectId) {
        $query = <<<SQL
        SELECT NAME
          FROM TASK
         WHERE PROJECT IN ($projectId)
           AND STATUS = 'A'
SQL;

        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query);
    }

    public function findTasksNamesFromTaskId($taskId) {
        $query = <<<SQL
           SELECT NAME
             FROM TASK
            WHERE ID IN ($taskId)
              AND STATUS = 'A'
SQL;

        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query);
    }



    public function validateRelationDevOps($projectId) {


		$query = <<<SQL
	  SELECT PRO.ID AS PROJECT_ID,SUB.ID AS SUB_RELEASE_ID,REL.ID AS RELEASE_ID,PRD.ID AS PRODUCT_ID
        FROM PROJECT PRO
        LEFT JOIN RELEASE REL ON (PRO.ID = REL.PROJECT_ID)
        LEFT JOIN SUBRELEASE SUB ON (SUB.ID = REL.PROJECT_ID)
        LEFT JOIN PRODUCT PRD ON (PRD.ID = REL.PROJECT_ID)
        WHERE PRO.ID = :PROJECT_ID
        AND (PRD.ID IS NOT NULL OR SUB.ID IS NOT NULL OR REL.ID IS NOT NULL)
SQL;
		$params = array(
			'PROJECT_ID' => $projectId
		);
		$result= $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		return !!(count($result));
	}

	/**
	 * @param $projectId
	 * @param $organization
	 * @return array
     */
	public function getDataSetForEditProjectBasic($projectId,$organization) {
		$query = <<<SQL
		     SELECT P.NAME,P.STATUS, P.ID, REP.NAME AS SPONSOR_NAME,ST.NAME AS "GROUP_NAME",PUP.NAME AS UPPER_PROJECT_NAME,
                    rtrim(xmlagg(xmlelement(e,REG.NAME ,',').extract('//text()') order by REG.NAME ).GetClobVal(),',') AS MANAGER_NAME,
                    rtrim(xmlagg(xmlelement(e,REG.ID ,',').extract('//text()') order by REG.NAME ).GetClobVal(),',') AS MANAGER,
                    (SELECT rtrim(xmlagg(xmlelement(e,SR.ID ,',').extract('//text()')).GetClobVal(),',')
                       FROM SUBRELEASE SR
                       WHERE P.ID = SR.PROJECT_ID
                         AND SR.IS_ACTIVE <> 'D') AS SUBRELEASE_ID,
				    P.PARENT AS UPPER_PROJECT,REP.ID AS SPONSOR,REP.ID AS BEGIN_SPONSOR,P.STRUCTURE AS "GROUP",
				    TO_CHAR(PUP.BEGIN_DATE,'YYYY-MM-DD') as BEGINPARENT,
				    CAL.DAY_HOURS,P.PROJECT_STATUS AS PROJECT_STATUS_ID,TS.LABEL as PROJECT_STATUS,TS.CONTROL AS PROJECT_STATUS_CONTROL,
				    TO_CHAR(P.DATE_EXCEPTION,'DD/MM/YYYY') AS DATE_EXCEPTION,PT.USETIMEPLANNING,TO_CHAR(P.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_AT,
                    PT.LABEL AS PROJECT_TYPE,PT.ID AS PROJECT_TYPE_ID,TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') AS "START",
                    P.DATE_EXCEPTION_TYPE AS DATE_EXCEPTION_ABBREVIATION ,RCR.NAME AS CREATED_BY ,PT.EXTERNALTYPE ,
                    TO_CHAR(P.END_DATE,'DD/MM/YYYY') AS "END",P.CLASSIFICATION AS PROJECT_CATEGORIES_ID, CL.LABEL AS PROJECT_CATEGORIES,
                    (CASE WHEN (P.TIME_PlANNED is null OR P.TIME_PlANNED = 0 )THEN NULL ELSE P.TIME_PlANNED END)AS TIME_PlANNED,
                    P.DURATION AS TOTAL_TIME,P.DESCRIPTION,RE.ID AS RELEASE_ID,PR.ID AS PRODUCT_ID,
                    DECODE(PRV.ID, NULL, NULL, PRV.VERSION || '.' || PRV.REVISION) AS VERSION_DEV,
                    PRV.ID AS PRODUCT_VERSION_DEV_ID, P.IS_LTS_REPLICATION
			   FROM PROJECT P
          LEFT JOIN RESOURCE_PROJECT PAR ON(P.ID = PAR.PROJECT and PAR.TYPE = 'S' )
          LEFT JOIN RESOURCE_PROJECT GER ON (P.ID = GER.PROJECT AND GER.TYPE = 'M' )
          LEFT JOIN RESOURCE_T REG ON(REG.ID = GER.RESOURCE_C AND REG.STATUS = 'A')
          LEFT JOIN RESOURCE_T REP ON(REP.ID = PAR.RESOURCE_C)
          LEFT JOIN RESOURCE_T RCR ON(RCR.ID = P.CREATED_BY)
          LEFT JOIN STRUCTURE ST ON(ST.ID = P.STRUCTURE)
          LEFT JOIN PROJECT PUP ON(PUP.ID = P.PARENT)
          LEFT JOIN CALENDAR CAL ON(CAL.ID = P.CALENDAR)
          LEFT JOIN PROJECT_TYPE PT ON(PT.ID = P.PROJECT_TYPE)
          LEFT JOIN CLASSIFICATION CL ON (CL.ID = P.CLASSIFICATION)
          LEFT JOIN TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS)
          LEFT JOIN RELEASE RE ON (P.ID = RE.PROJECT_ID AND RE.IS_ACTIVE <> 'D')
          LEFT JOIN PRODUCT PR ON (P.ID = PR.PROJECT_ID AND PR.IS_ACTIVE <> 'D')
          LEFT JOIN PRODUCT_VERSION PRV ON PRV.ID = P.PRODUCT_VERSION_ID
              WHERE P.ORGANIZATION = :organization
                AND P.ID = :project
           GROUP BY P.NAME,P.STATUS, P.ID, REP.NAME, ST.NAME ,
                    PUP.NAME, P.PARENT ,REP.ID,REP.ID ,P.STRUCTURE , PUP.BEGIN_DATE, CAL.DAY_HOURS,P.PROJECT_STATUS ,
                    TS.LABEL ,TS.CONTROL, P.DATE_EXCEPTION,PT.USETIMEPLANNING,P.CREATED_AT, PT.EXTERNALTYPE ,PT.LABEL ,PT.ID ,P.BEGIN_DATE,
                     P.DATE_EXCEPTION_TYPE,RCR.NAME, P.END_DATE,P.CLASSIFICATION , CL.LABEL ,P.TIME_PlANNED, P.DURATION ,
                     P.DESCRIPTION,RE.ID ,PR.ID, PRV.ID, PRV.VERSION, PRV.REVISION, P.IS_LTS_REPLICATION
SQL;
		$params = array(
			'organization' => $organization,
			'project' => $projectId
		);
		$connection = $this->getEntityManager()->getConnection();
		$result = $connection->fetchAll($query, $params);

        foreach ($result as &$project) {
            $project['MANAGER_NAME'] = Utilities::setSelectMultiInQuery($project['MANAGER_NAME']);
            $project['MANAGER'] = Utilities::setSelectMultiInQuery($project['MANAGER']);
            $project['BEGIN_MANAGER'] = $project['MANAGER'];
        }

        return $result;
	}

    /**
     * @param $project
     * @param $organization
     * @param int $userId
     * @return array
     */
	public function getDataSetForEditInfoMeeting($project,$organization,$userId = 0) {

		$query = <<<SQL
		  SELECT P.NAME,P.STATUS, P.ID,REG.NAME AS MANAGER,ST.NAME AS "GROUP_NAME",
				   PUP.NAME AS UPPER_PROJECT_NAME,GER.RESOURCE_C AS MANAGER_ID,
				   (CASE WHEN GER.RESOURCE_C = $userId OR GRP.RESOURCE_C = $userId THEN 'Y' ELSE 'N'END ) AS IS_MANAGER,
				   P.PARENT AS UPPER_PROJECT,P.STRUCTURE AS "GROUP",
				   TO_CHAR(PUP.BEGIN_DATE,'YYYY-MM-DD') as BEGINPARENT,REC.NAME AS CREATED_BY,
				   P.PROJECT_STATUS AS PROJECT_STATUS_ID,TS.LABEL as PROJECT_STATUS,TS.CONTROL AS PROJECT_STATUS_CONTROL,
				   TO_CHAR(P.DATE_EXCEPTION,'DD/MM/YYYY') AS DATE_EXCEPTION,PT.USETIMEPLANNING,
                   PT.LABEL AS PROJECT_TYPE,PT.ID AS PROJECT_TYPE_ID,TO_CHAR(TO_DATE(P.BEGIN_DATE),'DD/MM/YYYY') AS "START",
                   P.DATE_EXCEPTION_TYPE AS DATE_EXCEPTION_ABBREVIATION ,P.PARENT,PT.EXTERNALTYPE AS PROJECT_TYPE_CONTROL,
                   TO_CHAR(P.END_DATE,'DD/MM/YYYY') AS "END",
                   TO_CHAR(MEE.ISSUE_DATE_START,'DD/MM/YYYY') AS "ISSUE_PERIOD_start",
                   TO_CHAR(MEE.ISSUE_DATE_END,'DD/MM/YYYY') AS "ISSUE_PERIOD_end",
                   TO_CHAR(MEE.TASK_DATE_START,'DD/MM/YYYY') AS "TASK_PERIOD_start",
                   TO_CHAR(MEE.TASK_DATE_END,'DD/MM/YYYY') AS "TASK_PERIOD_end",
                   P.BEGIN_DATE AS "BEGIN_DATE",
                   TO_CHAR(P.BEGIN_DATE,'HH24:MI') AS "TIME",
                   MEE.ALLOW_SCHEDULE_FILTER,
                   P.DURATION AS TOTAL_TIME,P.DESCRIPTION,
                  ( SELECT LISTAGG(RES.ID, ',') WITHIN GROUP (ORDER BY RES.NAME)
                      FROM RESOURCE_T RES
                      JOIN RESOURCE_PROJECT REP ON( REP.RESOURCE_C = RES.ID
                                                    AND REP.PROJECT = :project
                                                    AND REP.TYPE = :workType)) AS "RESOURCE",
                  ( SELECT LISTAGG(RES.ID, ',') WITHIN GROUP (ORDER BY RES.NAME)
                      FROM RESOURCE_T RES
                      JOIN RESOURCE_PROJECT REP ON( REP.RESOURCE_C = RES.ID
                                                    AND REP.PROJECT = :project
                                                    AND REP.TYPE = :sponsorType)) AS "CLIENT",
                  ( SELECT LISTAGG(PRO.ID, ',') WITHIN GROUP (ORDER BY PRO.NAME)
                      FROM MEETING_PROJECT MPR
                      JOIN PROJECT PRO ON(PRO.ID = MPR.PROJECT_ID)
                     WHERE MPR.MEETING_ID = MEE.ID ) AS PROJECT_SPRINT,
                  ( SELECT LISTAGG(MTS.TASK_STATUS_ID, ',') WITHIN GROUP (ORDER BY MTS.ID)
                      FROM MEETING_TASK_STATUS MTS
                     WHERE MTS.MEETING_ID = MEE.ID AND MTS.TYPE = :typeTaskStatusTask) AS TASK_STATUS,
                  ( SELECT LISTAGG(LAB.ID, ',') WITHIN GROUP (ORDER BY LAB.DESCRIPTION )
                      FROM TASK TAS
                      JOIN TASK_LABEL TLA ON(TLA.TASK = TAS.ID)
                      JOIN LABEL LAB ON (LAB.ID = TLA.LABEL)
                     WHERE TAS.PROJECT = :project AND TAS.PARENT IS NULL  AND TLA.TYPE = :typeTaskLabelTask) AS TASK_LABELS,
                  ( SELECT LISTAGG(LAB.ID, ',') WITHIN GROUP (ORDER BY LAB.DESCRIPTION )
                      FROM TASK TAS
                      JOIN TASK_LABEL TLA ON(TLA.TASK = TAS.ID)
                      JOIN LABEL LAB ON (LAB.ID = TLA.LABEL)
                     WHERE TAS.PROJECT = :project AND TAS.PARENT IS NULL  AND TLA.TYPE = :typeTaskLabelIssue) AS ISSUE_LABELS,
                  ( SELECT LISTAGG(MTS.TASK_STATUS_ID, ',') WITHIN GROUP (ORDER BY MTS.ID)
                      FROM MEETING_TASK_STATUS MTS
                     WHERE MTS.MEETING_ID = MEE.ID AND MTS.TYPE = :typeTaskStatusIssue) AS ISSUE_STATUS
				FROM PROJECT P
				LEFT JOIN RESOURCE_PROJECT GER ON (P.ID = GER.PROJECT AND GER.TYPE = 'M' )
				LEFT JOIN RESOURCE_PROJECT GRP ON (P.PARENT = GRP.PROJECT AND GRP.TYPE = 'M' )
				LEFT JOIN RESOURCE_T REG ON(REG.ID = GER.RESOURCE_C)
				LEFT JOIN RESOURCE_T REC ON(REC.ID = P.CREATED_BY)
				LEFT JOIN STRUCTURE ST ON(ST.ID = P.STRUCTURE)
				LEFT JOIN PROJECT PUP ON(PUP.ID = P.PARENT)
                LEFT JOIN PROJECT_TYPE PT ON(PT.ID = P.PROJECT_TYPE)
                LEFT JOIN TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS)
                LEFT JOIN MEETING MEE ON (MEE.PROJECT_ID = P.ID)
				WHERE P.ORGANIZATION = :organization
				AND P.ID = :project
SQL;

        $params = array(
			'organization' => $organization,
			'project' => $project,
			'workType' => Entities\ResourceProject::WORK,
			'sponsorType' => Entities\ResourceProject::SPONSOR,
			'typeTaskStatusIssue' => Entities\MeetingTaskStatus::TYPE_ISSUE,
			'typeTaskStatusTask' => Entities\MeetingTaskStatus::TYPE_TASK,
			'typeTaskLabelTask' => Entities\TaskLabel::TYPE_TASK,
			'typeTaskLabelIssue' => Entities\TaskLabel::TYPE_ISSUE,
		);
        $connection = $this->getEntityManager()->getConnection();
        $results = $connection->fetchAll($query, $params);
        foreach ($results as &$result) {
            $result['BEGIN_DATE'] = explode(" ", $result['BEGIN_DATE'])[0];
        }
        return $results;
    }

	/**
	 * @param $project
	 * @return array
     */
	public function getDataSetForEditProjectCalendar($project) {

		$query = <<<SQL
			 SELECT CA.ID,CA.NAME,CA.DAYS,CA.DAY_HOURS/3600 as HOUR,CA.MONTHLY_DAYS,CA.WEEKLY_HOURS
             FROM CALENDAR CA
             JOIN PROJECT PR ON(PR.CALENDAR = CA.ID AND PR.ID = :project)
SQL;

		$params = array(
			'project' => $project
		);
		$connection = $this->getEntityManager()->getConnection();
		$dataSet = $connection->fetchAll($query, $params);
        foreach ($dataSet as &$data){
            $week = explode('|',$data['DAYS']);
            foreach($week as $weekDays){
                $data[$weekDays] = 'Y';
            }
        }
        return $dataSet;
	}

    public function getModelProject($organizationId,$userId,$filterString = '') {
		$query = <<<SQL
            SELECT *
              FROM (SELECT PRO.ID,PRO.NAME, (CASE WHEN PTY.EXTERNALTYPE = :typeModelRestricted THEN 'Y' ELSE 'N' END ) AS IS_RESTRICTED,
		                   PTY.LABEL AS PROJECT_TYPE_NME
                      FROM PROJECT PRO
                      JOIN PROJECT_TYPE PTY ON(PTY.ID = PRO.PROJECT_TYPE AND PTY.EXTERNALTYPE in(:typeModelRestricted,:typeModelFree))
                      JOIN TASK_STATUS PST ON (PST.ID = PRO.PROJECT_STATUS AND PST.CONTROL NOT IN (:controlCanceled, :controlConcluded))
                     WHERE PRO.ORGANIZATION = :organization
                       AND PRO.STATUS = :isActive
                       AND EXISTS(SELECT REP.ID FROM RESOURCE_PROJECT REP WHERE REP.PROJECT = PRO.ID AND REP.RESOURCE_C = :userId)
                  ORDER BY PRO.NAME)
             WHERE 1 = 1
         $filterString
SQL;

		$params = array(
			'organization' => $organizationId,
			'typeModelRestricted' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_RESTRICTED,
			'typeModelFree' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_FREE,
			'isActive' => Entities\Project::IS_ACTIVE,
			'userId' => $userId,
            'controlCanceled'    => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'controlConcluded'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
		);
		$dataSet = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        return $dataSet;
	}

  /**
   * @param $managerId
   * @return array
     */
  public function getFistProjectIdByManager($managerId)
  {
      $sql = <<<SQL
      SELECT P.ID
      FROM    PROJECT P
      INNER JOIN  RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
      WHERE RP.RESOURCE_C = :manager
      AND   P.STATUS <> 'D'
      AND   RP.TYPE = 'M'
SQL;
      $params = array(
          'manager' => $managerId
      );
      return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
  }

  /**
   * @param $managerId
   * @return array
     */
  public function getParentsProjectsByManagerId($managerId){
    $sql = <<<SQL
     SELECT P.PARENT AS ID
       FROM PROJECT P
      INNER JOIN RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
       LEFT JOIN TASK_STATUS PRS ON(PRS.ID = P.PROJECT_STATUS)
      WHERE RP.RESOURCE_C = :manager
        AND PRS.CONTROL NOT IN (:controlCanceled, :controlConcluded, :controlHomologated)
        AND P.STATUS <> 'D'
        AND RP.TYPE = 'M'
        AND P.PARENT IS NOT NULL
SQL;
    $params = array(
        'manager' => $managerId,
        'controlCanceled'    => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
        'controlConcluded'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
        'controlHomologated' => Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED,
    );

    return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
  }

  public function getParentProjectsByProjectIds($projectIds) {
    $parentProjects = $this->getParentProjectIdsByProjectIds($projectIds);
    return Utilities::getArrayFromArrayAssociative($parentProjects);
  }

	/**
	 * @param $projectIds
	 * @return array
     */
	public function getParentProjectIdsByProjectIds($projectIds){
		$sql = <<<SQL
			SELECT P.PARENT AS ID
			  FROM PROJECT P
			 WHERE P.ID IN (:PROJECT_IDS)
			   AND P.STATUS <> 'D'
               AND P.PARENT IS NOT NULL
			   AND P.PARENT NOT IN (:PROJECT_IDS)
SQL;
		$params = array(
			'PROJECT_IDS' => $projectIds
		);
        $paramTypes = array(
            'PROJECT_IDS' => Connection::PARAM_STR_ARRAY,
        );

		return $this->getEntityManager()->getConnection()->fetchAll($sql, $params, $paramTypes);
	}

    public function getManagersByProject($projectId){
		$sql = <<<SQL
            SELECT R.RESOURCE_C
            FROM RESOURCE_PROJECT R
			JOIN  PROJECT P ON (R.PROJECT = P.ID)
			JOIN RESOURCE_T RES ON (RES.ID = R.RESOURCE_C)
			WHERE P.ID = :projectId
			AND   P.STATUS <> 'D'
			AND   RES.STATUS = 'A'
			AND   RES.EMAIL IS NOT NULL
			AND   R.TYPE = 'M'
SQL;
		$params = array(
			'projectId' => $projectId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
	}

	/**
	 * @param $projectId
	 * @return array
     */
	public function getProjectLessId($projectId) {
		$query = <<<SQL
			SELECT ID
			FROM  PROJECT P
			WHERE P.PARENT = :project
SQL;

		$params = array(
			'project' => $projectId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	/**
	 * @param $projectsId
	 * @return array
     */
	public function getAllProjectLessId($projectsId) {
        return Utilities::getAllProjectLessId($projectsId);
	}
    /**
	 * @param $managerId
	 * @return array
     */
	public function getAllActiveProjectByManager($managerId) {
        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($managerId);
        $sql = <<<SQL
				SELECT P.ID
                  FROM PROJECT P
                  WHERE $stringProjectByManager
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($sql);
	}

	/**
	 * @param $managerId
	 * @return array
     */
	public function getFistStructureIdByManager($managerId){
		$sql = <<<SQL
			SELECT ST.ID
		  	FROM 		STRUCTURE ST
		  	INNER JOIN 	PERMISSION PE ON (PE.STRUCTURE = ST.ID)
		 	WHERE 	PE.RESOURCE_T = :manager
		   	AND 	ST.STATUS = 'A'
		   	AND 	PE.TYPE = 'M'
SQL;
		$params = array(
			'manager' => $managerId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
	}

	/**
	 * @param $structureId
	 * @return array
     */
	public function getStructureLessId($structureId) {
		$query = <<<SQL
			SELECT ST.ID AS STRUCTURE
			FROM STRUCTURE ST
			WHERE ST.PARENT = :STRUCTURE
SQL;

		$params = array(
			'STRUCTURE' => $structureId
            );
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	/**
	 * @param $structuresId
	 * @return array
     */
	public function getAllStructureLessId($structuresId) {
        return Utilities::getAllStructureLessId($structuresId);
	}

	public function getProjectIdByStructuresOrIds($userId) {

        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId,'PRO.ID');
        $sql = <<<SQL
		SELECT PRO.ID AS ID
         FROM PROJECT PRO
         JOIN TASK_STATUS TS ON (TS.ID = PRO.PROJECT_STATUS)
        WHERE $stringProjectByManager
          AND PRO.STATUS = :isActive
          AND TS.CONTROL NOT IN (:statusNotIn)
SQL;

        $params = array(
            'userId' => $userId,
            'isActive' => Entities\Project::IS_ACTIVE,
            'managerType' => Entities\ResourceProject::MANAGER,
            'statusNotIn' => array(
                Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
            ),
        );
        $paramTypes = array(
            'statusNotIn' => Connection::PARAM_STR_ARRAY,
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($sql, $params, $paramTypes);
        return  Utilities::getArrayFromArrayAssociative($result);
	}

    public function findCreateRecurrenceMeetingPlanning($meetingId,$beginDate, $beginTime) {
        $query = <<<SQL
      SELECT PRO.ID,PRO.NAME,TO_CHAR(PRO.BEGIN_DATE,'DD/MM/YYYY')|| ' às ' ||TO_CHAR(PRO.BEGIN_DATE,'HH24:MI') AS BEGIN_DATE
        FROM PROJECT PRO
        JOIN TASK_STATUS TST ON (TST.ID = PRO.PROJECT_STATUS AND TST.CONTROL NOT IN (:CANCELED_STATUS,:CONCLUDED_STATUS))
       WHERE PRO.PARENT = :PROJECT_ID
         AND PRO.STATUS = 'A'
         AND PRO.BEGIN_DATE <= TO_DATE('$beginDate $beginTime:00','DD/MM/YYYY HH24:MI:SS')
         ORDER  BY PRO.BEGIN_DATE
SQL;
        $params = array(
            'PROJECT_ID' => $meetingId,
            'CANCELED_STATUS' => Entities\Project::PROJECT_CONCLUDED_STATUS,
            'CONCLUDED_STATUS' => Entities\Project::PROJECT_CANCELED_STATUS,
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }

    /**
     * @param $projectId
     * @param $filters
     * @return array
     */
	public function findProjectLog($projectId,$filters) {
        $projects = $this->getAllProjectLessId(array(array('ID' => $projectId)));
        $allProjectsArray = Utilities::getArrayFromArrayAssociative($projects);
        $allProjectsArray = array_unique($allProjectsArray);
        $projectsIdsString = Utilities::getStringForArrayWithDefault($allProjectsArray,'0');
		$sql = <<<SQL
		SELECT * FROM(
			SELECT PL.ID ID,
            (CASE WHEN PL.LOG_TYPE LIKE 'INSERT' THEN 'Inclusão'
            WHEN PL.LOG_TYPE LIKE 'UPDATE' THEN 'Modificação'
            WHEN PL.LOG_TYPE LIKE 'DELETE' THEN 'Exclusão' END) LOG_TYPE,
            PL.LOG_TYPE  AS TYPE,
            T.ID || '_' || T.NAME TASK_NAME,
            P.ID || '_' || P.NAME PROJECT_NAME,
            (CASE WHEN PL.COLUMN_NAME LIKE 'DURATION' THEN 'Duração'
            WHEN PL.COLUMN_NAME LIKE 'ESTIMATED_BEGIN_DATE'  THEN 'Data de início tarefa'
            WHEN PL.COLUMN_NAME LIKE 'BEGIN_DATE'  THEN 'Data de início projeto'
            WHEN PL.COLUMN_NAME LIKE 'ESTIMATED_END_DATE' THEN 'Data de fim tarefa'
            WHEN PL.COLUMN_NAME LIKE 'END_DATE' THEN 'Data de fim tarefa'
            WHEN PL.TABLE_NAME LIKE 'ALLOCATION' THEN 'Recurso da tarefa'
            WHEN PL.COLUMN_NAME LIKE 'STATUS' AND PL.LOG_TYPE LIKE 'DELETE' THEN 'Exclusão de tarefa'
            WHEN PL.COLUMN_NAME LIKE 'STATUS' THEN 'Status'
            WHEN PL.COLUMN_NAME LIKE 'PROJECT_STATUS' THEN 'Status do projeto'
            WHEN PL.COLUMN_NAME LIKE 'PROJECT' THEN 'Tarefa movida'
            WHEN PL.COLUMN_NAME LIKE 'TIME_PLANNED' THEN 'Pontos Estimados'
            WHEN PL.TASK IS NULL THEN 'Criação de projeto'
            else 'Criação de tarefa' END) COLUMN_NAME,
            AlO.NAME ALLOCATION,
            (CASE WHEN PL.COLUMN_NAME LIKE 'DURATION' THEN TO_CHAR(TO_NUMBER(PL.OLD_VALUE)/3600)||'H'
              WHEN PL.COLUMN_NAME LIKE 'TIME_PLANNED' THEN TO_CHAR(TO_NUMBER(PL.OLD_VALUE))
              WHEN PL.COLUMN_NAME LIKE 'STATUS' AND PL.OLD_VALUE LIKE 'I' THEN 'Inativo'
              WHEN PL.COLUMN_NAME LIKE 'STATUS' AND PL.OLD_VALUE LIKE 'A' THEN 'Ativo'
              WHEN PL.COLUMN_NAME LIKE 'PROJECT' THEN (SELECT POLD.NAME
                FROM  PROJECT POLD WHERE POLD.ID = PL.OLD_VALUE)
              WHEN PL.COLUMN_NAME LIKE 'PROJECT_STATUS' THEN (SELECT TOLD.LABEL
                FROM  TASK_STATUS TOLD WHERE TOLD.ID = PL.OLD_VALUE)
              ELSE PL.OLD_VALUE END ) OLD_VALUE,
            (CASE WHEN PL.COLUMN_NAME LIKE 'DURATION' THEN TO_CHAR(TO_NUMBER(PL.NEW_VALUE)/3600)||'H'
              WHEN PL.COLUMN_NAME LIKE 'TIME_PLANNED' THEN TO_CHAR(TO_NUMBER(PL.NEW_VALUE))
              WHEN PL.COLUMN_NAME LIKE 'STATUS' AND PL.NEW_VALUE LIKE 'I' THEN 'Inativo'
              WHEN PL.COLUMN_NAME LIKE 'STATUS' AND PL.NEW_VALUE LIKE 'A' THEN 'Ativo'
              WHEN PL.COLUMN_NAME LIKE 'PROJECT' THEN (SELECT POLD.NAME
                FROM  PROJECT POLD WHERE POLD.ID = PL.NEW_VALUE)
              WHEN PL.COLUMN_NAME LIKE 'PROJECT_STATUS' THEN (SELECT TOLD.LABEL
                FROM TASK_STATUS TOLD WHERE TOLD.ID = PL.NEW_VALUE)
              ELSE PL.NEW_VALUE END ) NEW_VALUE,
            RES.NAME CREATED_BY,
            TO_CHAR(PL.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_DATE,
            PL.COLUMN_NAME COLUMN_NAME_LOG,
            PL.CREATED_AT AS ORDER_GRID,
            PL.CREATED_AT AS CREATED_AT
            FROM PROJECT_LOG PL
            LEFT JOIN RESOURCE_T RE ON(PL.RESOURCE_T = RE.ID)
            LEFT JOIN PROJECT P ON(PL.PROJECT = P.ID)
            LEFT JOIN TASK T ON(PL.TASK = T.ID)
            LEFT JOIN RESOURCE_T RES ON(PL.CREATED_BY = RES.ID)
            LEFT JOIN RESOURCE_T AlO ON(PL.RESOURCE_T = AlO.ID)
            WHERE PL.PROJECT IN ($projectsIdsString)) P
             WHERE 1=1
SQL;
		return $this->getEntityManager()->getConnection()->fetchAll($sql.$filters);
	}
    /**
	 * @param $userId
	 * @return array
     */
	public function findProjectByOrganization($userId) {
        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);
		$sql = <<<SQL
			SELECT DISTINCT TO_CHAR(P.BEGIN_DATE, 'DD/MM/YYYY') AS BEGIN_DATE, (CASE WHEN P.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS,
							TO_CHAR(P.END_DATE, 'DD/MM/YYYY') AS END_DATE, P.PERCENT_COMPLETED, P.PARENT, REP.NAME AS SPONSOR, P.NAME, P.ID,
							(CASE WHEN CA.DAY_HOURS > 0 THEN P.DURATION / CA.DAY_HOURS || 'D' ELSE P.DURATION / (28800) || 'D' END) AS DURATION
			FROM		PROJECT P
			LEFT JOIN   RESOURCE_PROJECT RP	 ON (RP.PROJECT = P.ID)
			LEFT JOIN   RESOURCE_T REP		  ON (REP.ID = RP.RESOURCE_C)
			LEFT JOIN   CALENDAR CA			 ON (CA.ID = P.CALENDAR)
			LEFT JOIN   STRUCTURE S			 ON (S.ID = P.STRUCTURE)
			WHERE $stringProjectByManager
			AND P.STATUS <> 'D'
			AND RP.TYPE = 'S'
			ORDER BY P.ID
SQL;
		$params = array(
			'userd' => $userId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
	}

	/*********************************************************
	 * Creates a piece of query which is concated to main query,
	 * filtering projects.
	 * @param $conditions
	 * @return string
	 *********************************************************/
	public function getFinalConditions($conditions) {
		$finalCondition = ' ';
		foreach ($conditions as $param) {
			$index = sizeof($param['value']);

			if(strcmp($param['columnName'], 'MANAGER_NAME') == 0 || strcmp($param['columnName'], 'SPONSOR_ID') == 0 ) {
				if (strcmp($param['columnName'], 'MANAGER_NAME') == 0) {
					$finalCondition .= ' AND REPG.ID IN (';
				} else {
					$finalCondition .= ' AND REPP.ID IN (';
				}
				if ($index > 1) {
					foreach ($param['value'] as $value) {
						$index--;
						$finalCondition .= $value;
						if ($index > 0) {
							$finalCondition .= ",";
						}

					}
				} else {
					$finalCondition .= $param['value'][0];
				}
				$finalCondition .= ")";
			} else {
				if(strcmp($param['operator'], 'LIKE') != 0) {
					$string = false;
					$finalCondition .= " AND P.".$param['columnName']." ".$param['operator']." (";
				} else {
					$string = true;
					$finalCondition .= " AND UPPER(P.".$param['columnName'].") ".$param['operator']." UPPER( '";
				}
				if($index > 1) {
					foreach ($param['value'] as $value) {
						$index --;
						if(strcmp($param['columnName'], 'STATUS') == 0) {
							$finalCondition .=" '".$value."'";
						} else {
							$finalCondition .=$value;
						}
						if($index > 0) {
							$finalCondition .=",";
						}
					}
				} else {
					if($string) {
						$finalCondition .= $param['value'];
					} else {
						$finalCondition .= "'".$param['value'][0]."'";
					}
				}
				if(!$string) {
					$finalCondition .= ")";
				}
				if($string) {
					$finalCondition .= "') ";
				}
			}
		}
		return $finalCondition;
	}

    /**
     * @param $userId
     * @param $conditions
     * @return array
     */
public function findMyProjectByUser($userId, $conditions) {

    $conditionString = Utilities::createQueryFiltered($conditions);
    $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
    $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);
	$sql = <<<SQL
		SELECT rtrim(xmlagg(xmlelement(e,MANAGER,',').extract('//text()') order by MANAGER).GetClobVal(),',') AS MANAGER,
		       ALLOWENDTASK,BEGIN_DATE, BEGIN_DATE_PROJECT,DURATION,END_DATE,END_DATE_PROJECT,ID,IN_CALC,NAME,
		       PARENT,PERCENT_COMPLETED,PROJECT_STATUS,PROJECT_STATUS_CONTROL,PROJECT_STATUS_NAME,PROJECT_TYPE,SPONSOR,
		       SPONSOR_ID,STATUS,STRUCTURE,TIME_PLANNED,USETIMEPLANNING, MODEL, EXCEED_MODEL_DATE, EXTERNALTYPE, VERSION_DEV, VERSION_DEV_ID
		  FROM (SELECT P.ID, P.NAME ,REPP.NAME AS SPONSOR , P.PARENT, REPG.ID AS MANAGER, PR.PROJECT_RELATIONSHIP AS MODEL,
                        DECODE(PRV.ID, NULL, NULL, PRV.VERSION || '.' || PRV.REVISION) AS VERSION_DEV,
                        PRV.ID AS VERSION_DEV_ID,
                       (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS BEGIN_DATE,
    				   (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.END_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS END_DATE,
    				   (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.IN_CALC ELSE  'N' END ) AS IN_CALC,
    				   (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN
                       (CASE WHEN (P.TIME_PlANNED is null OR P.TIME_PlANNED = 0 )THEN NULL ELSE P.TIME_PlANNED END) ELSE  NULL END )AS TIME_PlANNED,
                       (SELECT CASE WHEN  PRT.USETIMEPLANNING != 'Y' THEN 'N'
                                    WHEN TO_NUMBER(TO_CHAR(PRO.BEGIN_DATE,'YYYYMMDD')) NOT BETWEEN TO_NUMBER(TO_CHAR(PRM.BEGIN_DATE,'YYYYMMDD')) AND TO_NUMBER(TO_CHAR(PRM.END_DATE, 'YYYYMMDD'))
                                      OR TO_NUMBER(TO_CHAR(PRO.END_DATE,'YYYYMMDD')) NOT BETWEEN TO_NUMBER(TO_CHAR(PRM.BEGIN_DATE,'YYYYMMDD')) AND TO_NUMBER(TO_CHAR(PRM.END_DATE, 'YYYYMMDD'))
                                    THEN 'Y' ELSE 'N' END as EXCEED_MODEL_DATE
                            FROM PROJECT PRO
                            JOIN PROJECT_RELATIONSHIP PRR ON (PRO.ID = PRR.PROJECT AND PRR.TYPE = 'M')
                            JOIN PROJECT PRM ON (PRM.ID = PRR.PROJECT_RELATIONSHIP)
                            JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE WHERE PRO.ID = P.ID) AS EXCEED_MODEL_DATE,
                            (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.DURATION ELSE  NULL END )AS DURATION ,
							(CASE WHEN HAPROP.ID IS NULL THEN 'N' ELSE 'Y' END) AS HAS_APPROPRIATION,
							(CASE WHEN PT.ALLOWENDTASK = 'Y' THEN P.PERCENT_COMPLETED||'%' ELSE NULL END ) AS PERCENT_COMPLETED,
							P.BEGIN_DATE AS BEGIN_DATE_PROJECT, P.END_DATE AS END_DATE_PROJECT, PT.ALLOWENDTASK,
							PT.USETIMEPLANNING,P.PROJECT_STATUS,P.PROJECT_STATUS AS PROJECT_STATUS_ID,TS.LABEL AS PROJECT_STATUS_NAME,TS.CONTROL AS PROJECT_STATUS_CONTROL,
                            REPP.ID AS SPONSOR_ID,P.STRUCTURE,P.CLASSIFICATION,P.PROJECT_TYPE,P.PROJECT_TYPE AS PROJECT_TYPE_ID,P.STATUS,
                            TS.ID AS TASK_STATUS,P.ID AS PROJECT, PT.EXTERNALTYPE AS EXTERNALTYPE
						FROM		PROJECT P
						LEFT JOIN   RESOURCE_PROJECT PARP ON(P.ID = PARP.PROJECT AND PARP.TYPE = 'S')
						LEFT JOIN   RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID AND RP.TYPE = 'M')
						LEFT JOIN   RESOURCE_T REPG ON(REPG.ID = RP.RESOURCE_C)
						LEFT JOIN   RESOURCE_T REPP ON(REPP.ID = PARP.RESOURCE_C)
						LEFT JOIN   CALENDAR CA ON (CA.ID = P.CALENDAR)
					    LEFT JOIN   PROJECT_RELATIONSHIP PR ON (P.ID = PR.PROJECT AND PR.TYPE = 'G')
						LEFT JOIN   STRUCTURE S ON (S.ID = P.STRUCTURE)
						LEFT JOIN   PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
						LEFT JOIN	TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS AND TS.IS_PROJECT = 'Y')
                        LEFT JOIN   PRODUCT_VERSION PRV ON PRV.ID = P.PRODUCT_VERSION_ID
						LEFT JOIN	(	SELECT      P.ID
										FROM        APPROPRIATION A
										INNER JOIN  TASK T ON (T.ID = A.TASK)
										INNER JOIN  PROJECT P ON (P.ID = T.PROJECT)
										GROUP BY    P.ID
									) HAPROP ON (HAPROP.ID = P.ID)
						WHERE   $stringProjectByManager
						AND	 P.STATUS <> 'D'
						$filterTypeProject
						) P
               WHERE 1 = 1
            $conditionString
            GROUP BY ALLOWENDTASK,BEGIN_DATE, BEGIN_DATE_PROJECT,DURATION,END_DATE,END_DATE_PROJECT,ID,IN_CALC,NAME,
		       PARENT,PERCENT_COMPLETED,PROJECT_STATUS,PROJECT_STATUS_CONTROL,PROJECT_STATUS_NAME,PROJECT_TYPE,SPONSOR,
		       SPONSOR_ID,STATUS,STRUCTURE,TIME_PLANNED,USETIMEPLANNING, MODEL, EXCEED_MODEL_DATE, EXTERNALTYPE, VERSION_DEV, VERSION_DEV_ID
			ORDER BY P.NAME,P.BEGIN_DATE_PROJECT DESC, P.END_DATE_PROJECT DESC
SQL;
	return $this->getEntityManager()->getConnection()->fetchAll($sql);

	}

    public function findAllProjectByUser($userId, $projectId) {
        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $sql = <<<SQL
               SELECT P.ID, P.NAME, P.PARENT, TS.CONTROL,PT.LABEL AS PROJECT_TYPE_NAME, PT.ID AS PROJECT_TYPE_ID,
                      DECODE(PV.ID, NULL, NULL, PV.VERSION || '.' || PV.REVISION) AS VERSION_DEV, PT.EXTERNALTYPE
                 FROM PROJECT P
            LEFT JOIN PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
            LEFT JOIN TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS AND TS.IS_PROJECT = 'Y')
            LEFT JOIN PRODUCT_VERSION PV ON P.PRODUCT_VERSION_ID = PV.ID
                WHERE $stringProjectByManager
                  AND P.STATUS = :isActive
                  AND TS.CONTROL IN(:controlsCreateTask)
                  AND P.ID NOT IN ($projectId)
                $filterTypeProject
                ORDER BY P.NAME,P.BEGIN_DATE DESC, P.END_DATE DESC
SQL;

        $params = array(
            'isActive' => Entities\Project::IS_ACTIVE,
            'userId' => $userId,
            'controlsCreateTask' => array(
                Entities\TaskStatus::TASK_STATUS_CONTROL_PLANNING,
                Entities\TaskStatus::TASK_STATUS_CONTROL_PROGRESS,
            ),
            'managerType' => Entities\ResourceProject::MANAGER,

        );
        $paramTypes = array(
            'controlsCreateTask' => Connection::PARAM_STR_ARRAY,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params,$paramTypes);

	}

    public function findMyProjectByUserMeeting($organizationId, $conditions)
    {

        $stringConditions = Utilities::createQueryFiltered($conditions);
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT', true);
        $query = <<<SQL
		SELECT ID, NAME, PARENT,BEGIN_DATE,END_DATE,PROJECT_STATUS, PROJECT_STATUS_CONTROL,STATUS,LABEL
		          FROM (
		          SELECT * FROM (
		                SELECT P.ID, P.NAME, P.PARENT,P.BEGIN_DATE,P.END_DATE,TS.LABEL AS PROJECT_STATUS,
		                TS.CONTROL AS PROJECT_STATUS_CONTROL,P.STATUS,PT.LABEL
						FROM		PROJECT P
						LEFT JOIN   STRUCTURE S ON (S.ID = P.STRUCTURE)
						LEFT JOIN   PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
						LEFT JOIN	TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS AND TS.IS_PROJECT = 'Y')
						WHERE P.STATUS = 'A'
						AND P.ORGANIZATION = :organizationId
						$filterTypeProject)
						WHERE 1 = 1
				  $stringConditions ) P
			GROUP BY ID, NAME, PARENT,BEGIN_DATE,END_DATE,PROJECT_STATUS, PROJECT_STATUS_CONTROL,STATUS,LABEL
			ORDER BY P.NAME ,P.BEGIN_DATE DESC, P.END_DATE DESC
SQL;
        $params = array(
            'organizationId' => $organizationId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

    }

    public function findProjectProductAndMeeting($userId, $filterString = '')
    {
		$query = <<<SQL
		SELECT * FROM (
SELECT P.NAME , P.ID , PT.LABEL AS TYPE_NAME
  FROM PROJECT P
  JOIN	PROJECT_TYPE PT ON (P.PROJECT_TYPE = PT.ID)
    WHERE P.ID IN (SELECT P1.ID
                       FROM PROJECT P1, RESOURCE_PROJECT RP
                      WHERE P1.STATUS <> 'D'
                        AND P1.PROJECT_STATUS IN (SELECT TS.ID
                                                    FROM TASK_STATUS TS
                                                   WHERE TS.IS_PROJECT = 'Y'
                                                     AND TS.CONTROL NOT IN ('C', 'X'))
                        AND P1.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P','M'))
                        AND RP.PROJECT = P1.ID
                        AND RP.RESOURCE_C = :RESOURCE_T
                        AND RP.TYPE = 'M'
                      UNION
                     SELECT P.ID
                       FROM PROJECT P
                      WHERE P.STATUS <> 'D'
                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P'))
                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                   FROM TASK_STATUS TS
                                                  WHERE TS.IS_PROJECT = 'Y'
                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                        AND P.ID IN (SELECT PJ.ID
                                       FROM PROJECT PJ
                                      WHERE PJ.STATUS <> 'D'
                                        AND PJ.STRUCTURE IN (SELECT S.ID
                                                               FROM STRUCTURE S
                                                              WHERE S.ID IN (SELECT ST.ID
                                                                               FROM STRUCTURE ST, PERMISSION PE
                                                                              WHERE ST.ID = PE.STRUCTURE
                                                                                AND ST.STATUS <> 'D'
                                                                                AND PE.TYPE = 'M'
                                                                                AND PE.RESOURCE_T = :RESOURCE_T)
                                                                AND S.STATUS <> 'D'
                                                              UNION
                                                             SELECT S.ID
                                                               FROM STRUCTURE S
                                                              WHERE S.STATUS <> 'D'
                                                              START WITH S.PARENT IN (SELECT ST.ID
                                                                                        FROM STRUCTURE ST, PERMISSION PE
                                                                                       WHERE ST.ID = PE.STRUCTURE
                                                                                         AND ST.STATUS <> 'D'
                                                                                         AND PE.TYPE = 'M'
                                                                                         AND PE.RESOURCE_T = :RESOURCE_T)
                                                            CONNECT BY PRIOR S.ID = S.PARENT))
                      UNION
                     SELECT P.ID
                       FROM PROJECT P
                      WHERE P.STATUS <> 'D'
                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P'))
                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                   FROM TASK_STATUS TS
                                                  WHERE TS.IS_PROJECT = 'Y'
                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                      START WITH P.PARENT IN (SELECT S.ID
                                                FROM STRUCTURE S
                                               WHERE S.ID IN (SELECT ST.ID
                                                                FROM STRUCTURE ST, PERMISSION PE
                                                               WHERE ST.ID = PE.STRUCTURE
                                                                 AND ST.STATUS <> 'D'
                                                                 AND PE.TYPE = 'M'
                                                                 AND PE.RESOURCE_T = :RESOURCE_T)
                                                 AND S.STATUS <> 'D'
                                               UNION
                                              SELECT S.ID
                                                FROM STRUCTURE S
                                               WHERE S.STATUS <> 'D'
                                               START WITH S.PARENT IN (SELECT ST.ID
                                                                         FROM STRUCTURE ST, PERMISSION PE
                                                                        WHERE ST.ID = PE.STRUCTURE
                                                                          AND ST.STATUS <> 'D'
                                                                          AND PE.TYPE = 'M'
                                                                          AND PE.RESOURCE_T = :RESOURCE_T)
                                             CONNECT BY PRIOR S.ID = S.PARENT)
                    CONNECT BY PRIOR P.ID = P.PARENT) )
                    WHERE 1 = 1
                    $filterString

SQL;
        $params = array(
            'RESOURCE_T' => $userId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

    }

    /**
	 * @param $userId
	 * @return array
     */
public function findMyProjectByUserWithMeeting($userId, $conditions) {

    if(sizeof($conditions)>0){
        $finalCondition = $this->getFinalConditions($conditions);
    } else {
        $conditions = false;
    }

    $filterTypeProject = "	OR (PT.EXTERNALTYPE IN ('M','O') AND REPG.ID = $userId AND REPP.ID = $userId))";
	$sql = <<<SQL
		SELECT * FROM (SELECT DISTINCT P.ID, P.NAME,REPP.NAME AS SPONSOR , P.PARENT, REPG.ID AS MANAGER,
                            (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS BEGIN_DATE,
							(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.END_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS END_DATE,
							(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.IN_CALC ELSE  'N' END ) AS IN_CALC,
							(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN
                              (CASE WHEN (TIME_PlANNED is null OR TIME_PlANNED = 0 )THEN NULL ELSE P.TIME_PlANNED END)
                            ELSE  NULL END )AS TIME_PlANNED,
                            (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.DURATION ELSE  NULL END )AS DURATION   ,
							(CASE WHEN P.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS,
							(CASE WHEN HAPROP.ID IS NULL THEN 'N' ELSE 'Y' END) AS HAS_APPROPRIATION,
							(CASE WHEN PT.ALLOWENDTASK = 'Y' THEN P.PERCENT_COMPLETED||'%' ELSE NULL END ) AS PERCENT_COMPLETED,
							P.BEGIN_DATE AS BEGIN_DATE_PROJECT, P.END_DATE AS END_DATE_PROJECT, PT.ALLOWENDTASK,
							PT.USETIMEPLANNING,TS.LABEL AS PROJECT_STATUS
						FROM		PROJECT P
						LEFT JOIN   RESOURCE_PROJECT PARP ON(P.ID = PARP.PROJECT)
						LEFT JOIN   RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
						LEFT JOIN   RESOURCE_T REPG ON(REPG.ID = RP.RESOURCE_C)
						LEFT JOIN   RESOURCE_T REPP ON(REPP.ID = PARP.RESOURCE_C)
						LEFT JOIN   CALENDAR CA ON (CA.ID = P.CALENDAR)
						LEFT JOIN   STRUCTURE S ON (S.ID = P.STRUCTURE)
						LEFT JOIN   PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
						LEFT JOIN	TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS AND TS.IS_PROJECT = 'Y')
						LEFT JOIN	(	SELECT      P.ID
										FROM        APPROPRIATION A
										INNER JOIN  TASK T ON (T.ID = A.TASK)
										INNER JOIN  PROJECT P ON (P.ID = T.PROJECT)
										GROUP BY    P.ID
									) HAPROP ON (HAPROP.ID = P.ID)
						WHERE   (P.ID IN (SELECT ID FROM TABLE( PROJECT_BY_USER($userId))))
						AND	 P.STATUS <> 'D'
						AND RP.TYPE = 'M'
						AND	 (PARP.TYPE = 'S'
						$filterTypeProject
SQL;
	if($conditions) {
		$sql = $sql.$finalCondition;
	}
	return $this->getEntityManager()->getConnection()->fetchAll($sql.") P
			ORDER BY P.BEGIN_DATE_PROJECT DESC, P.END_DATE_PROJECT DESC");

	}

	/**
	 * @param $userId
	 * @return array
     */
	public function findPublicProjectByUser($userId) {

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
		$sql = <<<SQL
SELECT * FROM (SELECT DISTINCT P.ID, P.NAME,REPP.NAME AS SPONSOR , P.PARENT,
                            (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS BEGIN_DATE,
							(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.END_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS END_DATE,
							(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.IN_CALC ELSE 'N' END ) AS IN_CALC,
							(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN
                              (CASE WHEN (TIME_PlANNED is null OR TIME_PlANNED = 0 )THEN NULL ELSE P.TIME_PlANNED END)
                            ELSE  NULL END )AS TIME_PlANNED,
                            (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN
                                (CASE WHEN CA.DAY_HOURS > 0 THEN P.DURATION/CA.DAY_HOURS||'D' ELSE P.DURATION/(28800)||'D' END)
                            ELSE  NULL END )AS DURATION ,
							(CASE WHEN P.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS,
							(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.PERCENT_COMPLETED||'%' ELSE NULL END ) AS PERCENT_COMPLETED,
							P.BEGIN_DATE AS BEGIN_DATE_PROJECT, P.END_DATE AS END_DATE_PROJECT
						FROM		PROJECT P
						LEFT JOIN   RESOURCE_PROJECT PARP ON(P.ID = PARP.PROJECT)
						LEFT JOIN   RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
						LEFT JOIN   RESOURCE_T REPG ON(REPG.ID = RP.RESOURCE_C)
						LEFT JOIN   RESOURCE_T REPP ON(REPP.ID = PARP.RESOURCE_C)
						LEFT JOIN   CALENDAR CA ON(CA.ID = P.CALENDAR)
						LEFT JOIN   STRUCTURE S ON (S.ID = P.STRUCTURE)
						LEFT JOIN   PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
						 WHERE   (1 = 2)
						AND	 P.STATUS <> 'D'
						AND	 PARP.TYPE = 'S'
						$filterTypeProject
SQL;
		return $this->getEntityManager()->getConnection()->fetchAll($sql.") P
			ORDER BY P.BEGIN_DATE_PROJECT DESC, P.END_DATE_PROJECT DESC");
	}

	public function checkProjectHasTemplateParentRecursive($projectId){
        $query = <<<SQL
            SELECT P.ID, P.NAME, PT.EXTERNALTYPE, PT.LABEL
              FROM PROJECT_TYPE PT
              JOIN PROJECT P ON P.PROJECT_TYPE = PT.ID
             WHERE PT.EXTERNALTYPE IN ('Q', 'F')
        START WITH P.ID IN ($projectId)
  CONNECT BY PRIOR P.PARENT = P.ID
SQL;

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function getAllProjectParent($projectId) {

        $projectTreeKey = $this->find($projectId)->getTreeKey();


        $sql = <<<SQL
        SELECT P.ID, P.NAME, TS.CONTROL
        FROM PROJECT P
        LEFT JOIN PROJECT_TYPE PT ON (P.PROJECT_TYPE = PT.ID AND PT.EXTERNALTYPE = 'I')
        LEFT JOIN TASK_STATUS TS ON (P.PROJECT_STATUS = TS.ID)
        WHERE P.STATUS = 'A'
        AND (TS.CONTROL <> :projectConcludedStatus OR TS.CONTROL IS NULL)
        AND P.TREE_KEY LIKE '$projectTreeKey.%'
        ORDER BY PT.EXTERNALTYPE
SQL;
        $params = array(
            'projectConcludedStatus' => \Service\Project::PROJECT_CONCLUDED_STATUS
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);

	}

	/**
	 * @param $userId
	 * @return array
     */
	public function findProjectByUserPMA($userId) {

        $sql = <<<SQL
			SELECT DISTINCT TO_CHAR(P.END_DATE,'DD/MM/YYYY') AS END_DATE, P.PERCENT_COMPLETED||'%' as PERCENT_COMPLETED, REPP.NAME AS SPONSOR,
							(CASE WHEN CA.DAY_HOURS>0 THEN P.DURATION/CA.DAY_HOURS||'D' ELSE P.DURATION/(28800)||'D' END) AS DURATION, P.ID, P.NAME, P.PARENT,
							(CASE WHEN P.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS, TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') AS BEGIN_DATE
			FROM		PROJECT P
			LEFT JOIN   RESOURCE_PROJECT PARP 	ON (P.ID = PARP.PROJECT)
			LEFT JOIN   RESOURCE_PROJECT RP 	ON (RP.PROJECT = P.ID)
			LEFT JOIN   RESOURCE_T REPG 		ON (REPG.ID = RP.RESOURCE_C)
			LEFT JOIN   RESOURCE_T REPP 		ON (REPP.ID = PARP.RESOURCE_C)
			LEFT JOIN   CALENDAR CA 			ON (CA.ID = P.CALENDAR)
			LEFT JOIN   STRUCTURE S 			ON (S.ID = P.STRUCTURE)
			WHERE
			(    REPG.ID = :userd
			  OR (P.ID IN (SELECT ID FROM TABLE( PROJECT_BY_USER($userId))))
		    )
			AND	 	P.STATUS <> 'D'
			AND	 	PARP.TYPE = 'S'
			ORDER BY P.ID
SQL;
		$params = array(
			'userd' => $userId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
	}

    /**
     * @param $projectId
     * @param int $taskId
     * @param string $betweenDatesFilter
     * @param int $resourceId
     * @param array $allPrecursor
     * @return array
     */
	public function findAllAllocationDayProject($projectId,$taskId = 0,$betweenDatesFilter = '',$resourceId = 0,
                                                $allPrecursor = array(0)) {

        if(empty($allPrecursor)){
            $allPrecursorString = 0;
        }else{
            $allPrecursorString = implode(',',$allPrecursor);
        }
        $sql = <<<SQL
              SELECT  *
                FROM (SELECT DISTINCT DA.HOUR_TOTAL_DAY AS HOUR_TOTAL_DAY,TO_CHAR( DA.DAY, 'DD/MM/YYYY') AS  DAY,
                        TO_CHAR( DA.DAY, 'YYYYMMDD') AS  ORDERBY,TA.ID||'_'||TA.NAME AS NAME
                        FROM DAILY_ALLOCATION DA
                        JOIN TASK TA ON (TA.ID = DA.TASK AND TA.STATUS ='A' )
                        WHERE DA.PROJECT = :projectId
                        AND DA.RESOURCE_T = :resourceId
                        AND DA.TASK     <> :taskId
                        AND DA.TASK     NOT IN($allPrecursorString)
                 $betweenDatesFilter )
            ORDER BY ORDERBY DESC
SQL;
        $params = array(
			'projectId' => $projectId,
			'taskId' => $taskId,
			'resourceId' => $resourceId,
		);
        /**
         * feito porque o banco não conseque concatenar uma string muito grande
         */
        $allAllocationDay = $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
        $allAllocationDayPGroupByDay = array();
        foreach ($allAllocationDay as &$allocationDay) {
            $dayCurrent = null;
            foreach ($allAllocationDayPGroupByDay as &$allocationDayPGroupByDay){
                if($allocationDayPGroupByDay['DAY'] == $allocationDay['DAY']){
                    $dayCurrent = $allocationDayPGroupByDay;
                }
            };
            if(!$dayCurrent){
                $dayCurrent = array();
                $dayCurrent['DAY'] =  $allocationDay['DAY'];
                $dayCurrent['HOUR_TOTAL_DAY'] =  $allocationDay['HOUR_TOTAL_DAY'];
                $dayCurrent['NAME_TASKS'] =  $allocationDay['NAME'];
                $allAllocationDayPGroupByDay[] = $dayCurrent;
            }else{
                $dayCurrent['HOUR_TOTAL_DAY'] +=  $allocationDay['HOUR_TOTAL_DAY'];
                $dayCurrent['NAME_TASKS'] .=  ', '.$allocationDay['NAME'];
            }
            foreach ($allAllocationDayPGroupByDay as &$allocationDayPGroupByDay){
                if($allocationDayPGroupByDay['DAY'] == $dayCurrent['DAY']){
                    $allocationDayPGroupByDay = $dayCurrent;
                }
            }
        };
        return $allAllocationDayPGroupByDay;
	}

	/**
	 * @param $userId
	 * @return array
     */
	public function findAssociateProjectByUser($userId, $conditions) {
        $conditionString = Utilities::createQueryFiltered($conditions);
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);

		$sql = <<<SQL
        SELECT ID,ALLOWENDTASK,BEGIN_DATE,BEGIN_DATE_PROJECT,END_DATE,END_DATE_PROJECT,IN_CALC,
        rtrim(xmlagg(xmlelement(e,MANAGER,',').extract('//text()') order by MANAGER).GetClobVal(),',') AS MANAGER,
        NAME,PARENT,PERCENT_COMPLETED,PROJECT_STATUS,DURATION, PROJECT_STATUS_NAME,
        SPONSOR,STATUS_NAME,TIME_PLANNED,USETIMEPLANNING
        FROM
         (SELECT P.ID, P.NAME,REPP.NAME AS SPONSOR ,REPP.ID AS SPONSOR_ID , P.PARENT, REPG.ID AS MANAGER,
          (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS BEGIN_DATE,
          (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(P.END_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS END_DATE,
          (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.IN_CALC ELSE  'N' END ) AS IN_CALC,
          (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN
          (CASE WHEN (TIME_PlANNED is null OR TIME_PlANNED = 0 )THEN NULL ELSE P.TIME_PlANNED END) ELSE  NULL END )AS TIME_PlANNED,
          (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN P.DURATION ELSE  NULL END )AS DURATION , PT.ALLOWENDTASK,
          (CASE WHEN P.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS_NAME,
          (CASE WHEN PT.ALLOWENDTASK = 'Y' THEN P.PERCENT_COMPLETED||'%' ELSE NULL END ) AS PERCENT_COMPLETED,
          P.BEGIN_DATE AS BEGIN_DATE_PROJECT, P.END_DATE AS END_DATE_PROJECT, PT.USETIMEPLANNING,TS.ID AS PROJECT_STATUS,
          TS.LABEL AS PROJECT_STATUS_NAME,P.CLASSIFICATION,PT.ID AS PROJECT_TYPE,P.STATUS,P.STRUCTURE

		  FROM		  PROJECT P
          LEFT JOIN   RESOURCE_PROJECT PARP 	ON (P.ID = PARP.PROJECT AND PARP.TYPE = 'S')
          LEFT JOIN   RESOURCE_PROJECT RP 	    ON (RP.PROJECT = P.ID)
          LEFT JOIN   RESOURCE_T REPA 		    ON (REPA.ID = RP.RESOURCE_C)
          LEFT JOIN   RESOURCE_PROJECT REPB     ON (P.ID = REPB.PROJECT AND REPB.TYPE = 'M')
          LEFT JOIN   RESOURCE_T REPG 		    ON (REPG.ID = REPB.RESOURCE_C)
          LEFT JOIN   RESOURCE_T REPP 		    ON (REPP.ID = PARP.RESOURCE_C)
          LEFT JOIN   CALENDAR CA 			    ON (CA.ID = P.CALENDAR)
          LEFT JOIN   STRUCTURE S 			    ON (S.ID = P.STRUCTURE)
          LEFT JOIN   PROJECT_TYPE PT           ON (PT.ID = P.PROJECT_TYPE)
		  LEFT JOIN	  TASK_STATUS TS            ON (TS.ID = P.PROJECT_STATUS AND TS.IS_PROJECT = 'Y')

          WHERE 	  (REPA.ID = :userd)
          AND         P.ID NOT IN (SELECT ID FROM TABLE( PROJECT_BY_USER(:userd)))
          AND	 	  P.STATUS <> 'D'
          $filterTypeProject )
          WHERE 1=1
          $conditionString
          GROUP BY ID,ALLOWENDTASK,BEGIN_DATE,BEGIN_DATE_PROJECT,END_DATE,END_DATE_PROJECT,IN_CALC,
      NAME,PARENT,PERCENT_COMPLETED,PROJECT_STATUS,PROJECT_STATUS_NAME,SPONSOR,STATUS_NAME,TIME_PLANNED,USETIMEPLANNING,DURATION
			ORDER BY NAME, BEGIN_DATE_PROJECT DESC, END_DATE_PROJECT DESC
SQL;
		$params = array(
			'userd' => $userId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}

    public function findAllMeeting($organizationId, $conditions) {

        $conditionsString = Utilities::createQueryFiltered($conditions);

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT');
        $sql = <<<SQL
		SELECT * FROM (
    SELECT ID, NAME,PARENT,MANAGER,MANAGER_NAME,BEGIN_DATE,PROJECT_STATUS,PROJECT_STATUS_ID,PROJECT_STATUS_CONTROL,BEGIN_DATE_FILTER,TOTAL_TIME
        FROM (SELECT DISTINCT P.ID,
		                    (CASE WHEN P.PARENT IS NOT NULL THEN P.NAME||' - '||TO_CHAR(P.BEGIN_DATE,'DD')||' de '|| TO_CHAR(P.BEGIN_DATE,'Month')  ELSE  P.NAME END ) AS NAME,
		                     P.PARENT, REPG.ID AS MANAGER,REPG.NAME as MANAGER_NAME,P.BEGIN_DATE AS BEGIN_DATE_FILTER,TS.ID AS PROJECT_STATUS_ID,
                            (CASE WHEN P.PARENT IS NOT NULL THEN TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY')||' às '|| TO_CHAR(P.BEGIN_DATE,'HH24:MI') ELSE  NULL END ) AS BEGIN_DATE,
                            (CASE WHEN P.PARENT IS NOT NULL THEN TS.LABEL ELSE  NULL END ) AS PROJECT_STATUS,
                            (CASE WHEN P.PARENT IS NOT NULL THEN TS.CONTROL ELSE  NULL END ) AS PROJECT_STATUS_CONTROL,
                            (CASE WHEN P.PARENT IS NOT NULL THEN P.DURATION  ELSE  NULL END ) AS TOTAL_TIME
                            FROM		PROJECT P
						LEFT JOIN   RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID AND (RP.TYPE = 'M' OR RP.TYPE IS NULL))
						LEFT JOIN   RESOURCE_T REPG ON(REPG.ID = RP.RESOURCE_C)
						LEFT JOIN   CALENDAR CA ON (CA.ID = P.CALENDAR)
						LEFT JOIN   STRUCTURE S ON (S.ID = P.STRUCTURE)
						LEFT JOIN   PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
						LEFT JOIN	TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS AND TS.IS_PROJECT = 'Y')
						WHERE  P.STATUS <> 'D'
						$filterTypeProject
                        AND P.ORGANIZATION = $organizationId)
                        ) P
                WHERE 1=1
                $conditionsString
                GROUP BY ID, NAME,PARENT,MANAGER,MANAGER_NAME,BEGIN_DATE_FILTER,PROJECT_STATUS,PROJECT_STATUS_CONTROL,TOTAL_TIME,
                BEGIN_DATE,PROJECT_STATUS_ID
                ORDER BY P.NAME ASC
SQL;
        $meetings = $this->getEntityManager()->getConnection()->fetchAll($sql);
        foreach($meetings as &$meeting){
            $meeting['TOTAL_TIME'] = Utilities::secondsToTime($meeting['TOTAL_TIME']);
        }
        return $meetings;

    }

    public function findProductMeeting($projectIds, $name) {
        $filterName = '';
        if(isset($name)){
            $filterName = ($name != "%%")  ? "AND ( UPPER(PRO.NAME) LIKE UPPER('".$name."'))" : "";
        }
        $projectIds = Utilities::arrayAssociativeToArray($projectIds);
        $sql = <<<SQL
     SELECT PRO.ID AS ID,PRO.NAME,PRO.PARENT, PRO.PRODUCT_TYPE
         FROM PROJECT PRO
         JOIN PROJECT_TYPE PRT ON (PRT.ID = PRO.PROJECT_TYPE AND EXTERNALTYPE = :typeProjectControlProduct)
         JOIN PROJECT PIS ON PIS.PARENT = PRO.ID
         JOIN PROJECT_TYPE PRI ON PRI.ID = PIS.PROJECT_TYPE AND PRI.EXTERNALTYPE = :issueType
        WHERE PRO.ID IN ( SELECT P.ID
                               FROM PROJECT P
                              WHERE P.ID IN (:projectIds)
                                AND P.STATUS = :isActive
                              UNION ALL
                             SELECT P.ID
                               FROM PROJECT P
                              WHERE P.STATUS = :isActive
                              START WITH P.PARENT IN (:projectIds)
                            CONNECT BY PRIOR P.ID = P.PARENT)
          AND PRO.STATUS = :isActive
          AND PIS.STATUS = :isActive
          AND PRO.PRODUCT_TYPE <> :productTypeManager

          $filterName

     ORDER BY PRO.NAME

SQL;
        $params = array(
            'projectIds' => $projectIds,
            'isActive' => Entities\Project::IS_ACTIVE,
            'typeProjectControlProduct' => Entities\ProjectType::TYPE_PROJECT_CONTROL_PRODUCT,
            'productTypeManager' => Entities\ProjectType::TYPE_PRODUCT_CONTROL_MANAGEMENT,
            'issueType' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE
        );
        $paramTypes = array(
            'projectIds' => Connection::PARAM_STR_ARRAY,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params, $paramTypes);
    }

    public function findProjectMeeting($projectIds,$userId,$conditionsString) {
        $allProjectsArray = $this->getProjectIdByStructuresOrIds($userId);
        $projectIds = array_intersect($allProjectsArray,$projectIds);
        $allProjectsString = Utilities::fixBugWhereINQuery($projectIds,'P.ID');
        $sql = <<<SQL
        SELECT  * FROM (
        SELECT PRO.ID AS ID,PRO.NAME,PRO.PARENT,PRO.PROJECT_STATUS,PRO.PROJECT_TYPE
              FROM PROJECT PRO
              JOIN TASK_STATUS TS ON (TS.ID = PRO.PROJECT_STATUS)
         LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
             WHERE PRO.ID IN ( SELECT P.ID
                                 FROM PROJECT P
                                WHERE P.STATUS = :isActive
                           START WITH ($allProjectsString)
                              CONNECT BY PRIOR P.ID = P.PARENT)
               AND PRO.STATUS = :isActive
               AND TS.CONTROL NOT IN (:statusNotIn)
               AND PT.EXTERNALTYPE NOT IN (:timeLineControl)
          ORDER BY PRO.NAME)
          WHERE 1 = 1
          $conditionsString
SQL;
        $params = array(
            'isActive' => Entities\Project::IS_ACTIVE,
            'statusNotIn' => array(
                Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
            ),
            'userId' => $userId,
            'managerType' => Entities\ResourceProject::MANAGER,
            'timeLineControl' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
        );
        $paramTypes = array(
            'statusNotIn' => Connection::PARAM_STR_ARRAY,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params, $paramTypes);

    }


    /**
	 * @param $structureId
	 * @param $userId
	 * @return array
     */
	protected function findProjectsByRecursiveStructure($structureId, $userId){
		$sql = <<<SQL
			SELECT S.ID
				FROM		STRUCTURE S
				LEFT JOIN   HIERARCHY H ON H.STRUCTURE = S.ID
				WHERE H.PARENT = :parent
SQL;
		$params = array(
			'parent' => $structureId
		);

		$children = $this->getEntityManager()->getConnection()->fetchAll($sql, $params);

		$projects = array();
		if(!empty($children)){
			foreach ($children as $child) {
				$sql = <<<SQL
					SELECT DISTINCT P.ID, P.NAME,BPP.NAME AS SPONSOR , P.PARENT,TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') AS BEGIN_DATE,
							TO_CHAR(P.END_DATE,'DD/MM/YYYY') AS END_DATE,
							(CASE WHEN CA.DAY_HOURS>0 THEN P.DURATION/CA.DAY_HOURS||'D' ELSE P.DURATION/24||'D' END) AS DURATION ,
							(CASE WHEN P.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS,1 as PERCENT_COMPLETED
						FROM		PROJECT P
						LEFT JOIN   RESOURCE_PROJECT PARP ON(P.ID = PARP.PROJECT)
						LEFT JOIN   RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
						LEFT JOIN   RESOURCE_T REPG ON(REPG.ID = RP.RESOURCE_C)
						LEFT JOIN   RESOURCE_T REPP ON(REPP.ID = PARP.RESOURCE_C)
						LEFT JOIN   BUSINESS_PARTNER BPP ON(BPP.ID = REPP.BUSINESS_PARTNER)
						LEFT JOIN   CALENDAR CA ON(CA.CALENDAR_ENTITY = P.CALENDAR_ENTITY)
						LEFT JOIN   STRUCTURE S ON (S.ID = P.STRUCTURE)
						WHERE   P.STRUCTURE = :structure
						AND	 P.STATUS <> 'D'
						AND	 PARP.TYPE = 'S'
						AND	 REPG.BUSINESS_PARTNER <> :user
						ORDER BY P.ID
SQL;
				$params = array(
					'structure' => $child['ID'],
					'user' => $userId
				);
				$projects = array_merge($projects, $this->getEntityManager()->getConnection()->fetchAll($sql, $params),
										$this->findProjectsByRecursiveStructure($child['ID'], $userId));
			}
		}
		return $projects;
	}

	/**
	 * @param $userId
	 * @return array
     */
	public function findProjectByUser($userId) {

		$recursiveProjects = array();

		$sql = <<<SQL
			SELECT S.ID
				FROM STRUCTURE S
				WHERE S.BUSINESS_PARTNER = :user
SQL;

		$params = array(
			'user' => $userId
		);

		$structures = $this->getEntityManager()->getConnection()->fetchAll($sql,$params);

		foreach ($structures as $structure) {
			$recursiveProjects = array_merge($recursiveProjects, $this->findProjectsByRecursiveStructure($structure['ID'], $userId));
		}

		$sql = <<<SQL
SELECT DISTINCT P.ID, P.NAME,BPP.NAME AS SPONSOR , P.PARENT,TO_CHAR(P.BEGIN_DATE,'DD/MM/YYYY') AS BEGIN_DATE, TO_CHAR(P.END_DATE,'DD/MM/YYYY') AS END_DATE,
				(CASE WHEN CA.DAY_HOURS>0 THEN P.DURATION/CA.DAY_HOURS||'D' ELSE P.DURATION/28800||'D' END) AS DURATION ,
				(CASE WHEN P.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS,1 as PERCENT_COMPLETED
				FROM PROJECT P
				LEFT JOIN RESOURCE_PROJECT PARP ON(P.ID = PARP.PROJECT)
				LEFT JOIN RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
				LEFT JOIN RESOURCE_T REPG ON(REPG.ID = RP.RESOURCE_C)
				LEFT JOIN RESOURCE_T REPP ON(REPP.ID = PARP.RESOURCE_C)
				LEFT JOIN BUSINESS_PARTNER BPP ON(BPP.ID = REPP.BUSINESS_PARTNER)
				LEFT JOIN CALENDAR CA ON(CA.CALENDAR_ENTITY = P.CALENDAR_ENTITY)
				LEFT JOIN STRUCTURE S ON (S.ID = P.STRUCTURE)
				WHERE   P.STATUS <> 'D'
				AND	 PARP.TYPE = 'S'
				AND (
						(
							REPG.BUSINESS_PARTNER = :user
						)
						OR S.BUSINESS_PARTNER = :user
					)
				ORDER BY P.ID
SQL;

		return array_merge($this->getEntityManager()->getConnection()->fetchAll($sql,$params), $recursiveProjects);
	}

    public function findResourceProjectByProjectAndType($projectId,$type){
        $query = <<<SQL
              SELECT RES.ID
                FROM RESOURCE_T RES
                JOIN RESOURCE_PROJECT REP ON(REP.RESOURCE_C = RES.ID )
               WHERE REP.PROJECT = :PROJECT_ID AND REP.TYPE = :TYPE
               ORDER BY RES.NAME
SQL;
        $params = array(
            'PROJECT_ID' => $projectId,
            'TYPE' => $type,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findManagerByProject($projectId){
        return $this->findResourceProjectByProjectAndType($projectId,Entities\ResourceProject::MANAGER);
    }
    public function findSponsorByProject($projectId){
        return $this->findResourceProjectByProjectAndType($projectId,Entities\ResourceProject::SPONSOR);
    }

    public function findProjectParent($organizationId, $filterString = '') {
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);

        $query = <<<SQL
            SELECT *
              FROM (SELECT P.ID, P.NAME, P.PARENT,PT.LABEL AS PROJECT_TYPE_NAME, ST.NAME AS STRUCTURE
                      FROM PROJECT P
                      JOIN STRUCTURE ST ON(ST.ID = P.STRUCTURE)
                 LEFT JOIN PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
                     WHERE P.ORGANIZATION = :ORGANIZATION
                           $filterTypeProject
                       AND P.STATUS = 'A'
                       AND P.PROJECT_STATUS NOT IN (SELECT TS.ID
                                                 FROM TASK_STATUS TS
                                                WHERE TS.CONTROL IN (:CONTROL_CANCELED, :CONTROL_CONCLUDED, :CONTROL_HOMOLOGATED)))
            WHERE 1=1
            $filterString
            ORDER BY NAME
SQL;
        $params = array(
            'ORGANIZATION' => $organizationId,
            'CONTROL_CANCELED'    => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'CONTROL_CONCLUDED'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'CONTROL_HOMOLOGATED' =>  Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED,
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
	}

	public function findMnemonicosValuesRTR($projectId, $date, $userId)
	{

	    $query = <<<SQL
        		SELECT NULL AS PROPOSTA, NULL AS SERVICE_NAME, NULL AS CUSTOMER_REPRESENTATIVE, NULL AS BILLED_DESLOC,
                	   PROJECT_ID, PROJECT_NAME, DATE_CDA, TOTAL, CUSTOMER_NAME, CUSTOMER_ID, USER_NAME, UNBILLED,
                	   TRUNC(TOTAL/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(TOTAL/60),60)),2,2) AS TOTAL_WORK,
                	   CASE WHEN UNBILLED IS NULL
                	   		THEN  TRUNC((TOTAL)/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC((TOTAL)/60),60)),2,2)
                		    ELSE TRUNC((TOTAL - UNBILLED)/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC((TOTAL - UNBILLED)/60),60)),2,2)
                		END AS TOTAL_BILLED,
                	   CASE WHEN UNBILLED IS NULL
                	   		THEN '0:00'
                	   		ELSE TRUNC(UNBILLED / 3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(UNBILLED / 60),60)),2,2)
                	   	END AS UNBILLED_TOTAL
                  FROM (SELECT P.ID AS PROJECT_ID, P.NAME AS PROJECT_NAME, TO_CHAR(APP.BEGIN_HOUR,'DD/MM/YYYY') AS DATE_CDA,
                        	   SUM(APP.TOTAL) AS TOTAL, REST.NAME AS CUSTOMER_NAME, REST.ID AS CUSTOMER_ID,
                			   (SELECT NAME FROM RESOURCE_T WHERE ID = :userid) AS USER_NAME,
                			   (SELECT CASE WHEN SUM(APP.TOTAL) IS NULL
                			                THEN 0
                    						ELSE SUM(APP.TOTAL)
                    					END AS UNBILLED_TOTAL
                    			  FROM APPROPRIATION APP
                    			  JOIN TASK TAS ON (TAS.ID = APP.TASK)
								  JOIN PROJECT P ON (P.ID = TAS.PROJECT)
								  JOIN PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
								 WHERE PT.EXTERNALTYPE = 'C'
								   AND TAS.STATUS = 'A'
								   AND P.STATUS = 'A'
								   AND APP.PROJECT_ID = :projectid
								   AND TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :date
							  GROUP BY P.ID) AS UNBILLED,
                    		   NULL AS PROPOSTA, NULL AS SERVICE_NAME, NULL AS CUSTOMER_REPRESENTATIVE, NULL AS BILLED_DESLOC
                 		  FROM PROJECT P
                		  JOIN TASK TAS ON TAS.PROJECT = P.ID
                		  JOIN APPROPRIATION APP ON APP.TASK = TAS.ID
                		  JOIN RESOURCE_PROJECT RESP ON RESP.PROJECT = P.ID AND RESP.TYPE = 'S'
                		  JOIN RESOURCE_T REST ON REST.ID = RESP.RESOURCE_C
                		 WHERE APP.RESOURCE_T = :userid AND TAS.STATUS = 'A' AND P.ID = :projectid AND TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :date
                	  GROUP BY TO_CHAR(APP.BEGIN_HOUR,'DD/MM/YYYY'), P.ID, P.NAME, REST.NAME, REST.ID)
            WHERE 1=1
SQL;
        $params = array(
            'userid' => $userId,
            'projectid' => $projectId,
            'date' => $date
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function findProject($userId)
	{
	    $query = <<<SQL
	   SELECT P.ID, P.NAME, P.PARENT FROM PROJECT P
         JOIN PROJECT_TYPE PT ON PT.ID = P.PROJECT_TYPE AND PT.EXTERNALTYPE NOT IN (:MODEL_RESTRICTED, :MODEL_FREE,:MEETING,:MEETING_OCCURRENCES,:TIME_LINE)
         JOIN RESOURCE_PROJECT RESP ON RESP.PROJECT = P.ID
         JOIN RESOURCE_T REST ON REST.ID = RESP.RESOURCE_C
        WHERE REST.ID = :userId
          AND P.STATUS = 'A'
          ORDER BY USETIMEPLANNING DESC, P.END_DATE DESC
SQL;

        $params = array(
                'userId' => $userId,
                'MODEL_RESTRICTED'      => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_RESTRICTED,
                'MODEL_FREE'            => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_FREE,
                'MEETING'               => Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                'MEETING_OCCURRENCES'   => Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                'TIME_LINE'             => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
            );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

	}

	public function findTaskMnemonicsRTR($projectId, $date, $userId)
	{

	    $query = <<<SQL

        SELECT TAS.NAME AS TASK_NAME,TAS.DESCRIPTION AS TASK_DESCRIPTION, TAS.PERCENT_COMPLETED AS TASK_CONCLUSION,
        	   TRUNC(SUM(APP.TOTAL)/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(SUM(APP.TOTAL)/60),60)),2,2) AS TASK_TIME,
        	   (SELECT TRUNC(SUM(APP.TOTAL)/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(SUM(APP.TOTAL)/60),60)),2,2) AS UNBILLED_TOTAL
				 FROM APPROPRIATION APP
				 JOIN TASK TAS ON (TAS.ID = APP.TASK)
				 LEFT JOIN PROJECT P ON (P.ID = TAS.PROJECT)
				 LEFT JOIN PROJECT_TYPE PT ON (PT.ID = P.PROJECT_TYPE)
				 WHERE PT.EXTERNALTYPE = 'C' AND APP.PROJECT_ID = :projectid) AS UNBILLED_TOTAL,
				 TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') AS DATE_CDA
		 FROM PROJECT P
		 JOIN TASK TAS ON TAS.PROJECT = P.ID
		 JOIN APPROPRIATION APP ON APP.TASK = TAS.ID
		 JOIN RESOURCE_PROJECT RESP ON RESP.PROJECT = P.ID
		 						   AND RESP.TYPE = 'S'
		 JOIN RESOURCE_T REST ON REST.ID = RESP.RESOURCE_C
		WHERE APP.RESOURCE_T = :userid
		  AND TAS.STATUS = 'A'
		  AND P.ID = :projectid
		  AND TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :date
		GROUP BY TAS.NAME, P.NAME, REST.NAME, TAS.DESCRIPTION,  TAS.PERCENT_COMPLETED, P.ID, REST.ID, TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY')
SQL;
        $params = array(
            'userid' => $userId,
            'projectid' => $projectId,
            'date' => $date
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function findReportCDA($userId, $filterString)
	{

	    $query = <<<SQL
	    SELECT TREE_ID, PARENT_ID, PROJECT_ID, SPONSOR_NAME,SPONSOR_ID, PROJECT_NAME, BEGIN_HOUR AS NAME, BEGIN_HOUR,
	     	   BILLABLE_HOURS, 'N' AS IS_CDA
	        FROM (
              SELECT TO_CHAR(APP.BEGIN_HOUR,'DD/MM/YYYY')|| P.ID AS TREE_ID, P.ID AS PROJECT_ID, P.NAME AS PROJECT_NAME, TO_CHAR(APP.BEGIN_HOUR,'DD/MM/YYYY') AS BEGIN_HOUR,
                TRUNC(SUM(APP.TOTAL)/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(SUM(APP.TOTAL)/60),60)),2,2) AS BILLABLE_HOURS,
                REST.NAME AS SPONSOR_NAME, TO_CHAR(REST.ID) AS SPONSOR_ID, P.ID || '_P' AS PARENT_ID
                 FROM PROJECT P
                JOIN TASK TAS ON TAS.PROJECT = P.ID
                JOIN APPROPRIATION APP ON APP.TASK = TAS.ID
                JOIN RESOURCE_PROJECT RESP ON RESP.PROJECT = P.ID AND RESP.TYPE = 'S'
                JOIN RESOURCE_T REST ON REST.ID = RESP.RESOURCE_C
                WHERE APP.RESOURCE_T = :USERID AND TAS.STATUS = 'A'
                GROUP BY TO_CHAR(APP.BEGIN_HOUR,'DD/MM/YYYY'), P.ID, P.NAME, REST.NAME, REST.ID
                 )
            WHERE 1 = 1
            $filterString
SQL;

        $query = "
            SELECT PARENT_ID AS TREE_ID, NULL AS PARENT_ID, NULL AS PROJECT_ID,  SPONSOR_NAME, NULL AS SPONSOR_ID,
             	   NULL AS PROJECT_NAME, PROJECT_NAME AS NAME, NULL AS BEGIN_HOUR, NULL AS BILLABLE_HOURS, IS_CDA
            FROM ( $query )
             GROUP BY PARENT_ID, SPONSOR_NAME, PROJECT_NAME, BEGIN_HOUR, IS_CDA
            UNION
            $query";

		$queryCda = <<<SQL
			 SELECT TREE_ID, PARENT_ID, PROJECT_ID, SPONSOR_NAME, SPONSOR_ID, PROJECT_NAME,
			  		BEGIN_HOUR AS NAME, BEGIN_HOUR,
			  		(SELECT HORADEC_HORACHAR(SUM(BILLABLE_HOURS)) FROM DUAL) AS BILLABLE_HOURS, 'Y' AS IS_CDA
			   FROM (SELECT TO_CHAR(AG.DTAPROPRIACAO,'DD/MM/YYYY') || CO.NRSEQCONTRATO AS TREE_ID, CO.NRSEQCONTRATO || '_' || 'P' AS PARENT_ID,
							CO.NRSEQCONTRATO AS PROJECT_ID, CL.NMFANTCLIE AS SPONSOR_NAME, CL.CDCLIENTE AS SPONSOR_ID,
							CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS PROJECT_NAME, TO_CHAR(AG.DTAPROPRIACAO,'DD/MM/YYYY') AS BEGIN_HOUR,
							SUM(AG.QTHORASAPROPRIADAS) AS BILLABLE_HOURS
					   FROM RESOURCE_T            RE
					   JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
					   JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
					   JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
					   JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT

					   JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
					   JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
					   JOIN CLIENTE               CL ON CL.NRPARCNEGOCIO = CO.NRPARCNEGOCIO

					  WHERE RE.ID = :USERID
				   GROUP BY TO_CHAR(AG.DTAPROPRIACAO,'DD/MM/YYYY'), CO.NRSEQCONTRATO, CO.NRCONTRATOFCT, CO.DSREFERENCIACONT,
							 CL.NMFANTCLIE, CL.CDCLIENTE )
			WHERE 1 = 1
			$filterString
SQL;

		$queryCda = "
            SELECT PARENT_ID AS TREE_ID, NULL AS PARENT_ID, NULL AS PROJECT_ID,  SPONSOR_NAME, NULL AS SPONSOR_ID, NULL AS PROJECT_NAME,
                    PROJECT_NAME AS NAME, NULL AS BEGIN_HOUR, NULL AS BILLABLE_HOURS, IS_CDA
              FROM ( $queryCda )
             GROUP BY PARENT_ID, SPONSOR_NAME, PROJECT_NAME, BEGIN_HOUR, IS_CDA
            UNION
            $queryCda";

		$query = "
			$query
			UNION
			$queryCda
		";

        $params = array(
            'USERID' => $userId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	/**
	 * @param $structureId
	 * @return array
     */
	public function findProjectbyStructure($structureId) {
		$query =
				"SELECT P.ID FROM PROJECT p
				WHERE p.STRUCTURE = :structure
				AND p.STATUS <> 'D'";

		$params = array(
			'structure' => $structureId
		);
		$connection = $this->getEntityManager()->getConnection();
		return $connection->fetchAll($query, $params);

	}

	/**
	 * @param $userId
	 * @return string
	 */
	public function getAllStructureByManagerInString($userId)
	{
		$structure = $this->getFistStructureIdByManager($userId);
		$allStructure = $this->getAllStructureLessId($structure);
        $allStructuresArray = Utilities::getArrayFromArrayAssociative($allStructure);
        $allStructuresString = Utilities::getStringForArrayWithDefault($allStructuresArray,'0');
        return $allStructuresString;
	}

    /**
	 * @param $userId
	 * @return string
	 */
	public function getAllProjectAndHisParentsByManager($userId){
        $parentProjects = $this->getParentsProjectsByManagerId($userId);
        $parentProjectsArray = Utilities::getArrayFromArrayAssociative($parentProjects);
        $allProject = $this->getAllActiveProjectByManager($userId);
        $allProjectsArray = Utilities::getArrayFromArrayAssociative($allProject);
        $allProjectsArray = array_merge($allProjectsArray, $parentProjectsArray);
        return array_unique($allProjectsArray);
	}

	public function getManagerProjectsId($managerId) {
		$projects = $this->getFistProjectIdByManager($managerId);
		if(sizeof(($projects)) > 0) {
			$projectsIds = [];
			foreach ($projects as $project) {
				array_push($projectsIds, $project['ID']);
			}
			$projectsIdsString = implode(",",$projectsIds);
		} else {
			$projectsIdsString = "0";
		}

		return $projectsIdsString;
	}

    public function getChildrenProjectsId($projectIds) {
        $projects = $this->getAllProjectLessId($projectIds);
        $allProjectsArray = Utilities::getArrayFromArrayAssociative($projects);
        $allProjectsArray = array_unique($allProjectsArray);
        $allProjectsString = Utilities::getStringForArrayWithDefault($allProjectsArray,'0');
        return $allProjectsString;
    }


    public function findProducts($organizationId, $projectId = 0, $filterName = false, $filterOperator = false, $filterValue= false, $filter = false, $filterUserAcess = ''){
        $filterName ? $filterString = "AND PRD.".$filterName." ".$filterOperator." '".$filterValue."'" : $filterString = "";
        $query = <<<SQL
        SELECT *
          FROM (SELECT P.*,PRO.ID AS DEVOPS_ID, OMP.IDPERCRIARCHAMADO
    		      FROM (SELECT PRD.ID, PRD.NAME, PRD.ID AS BOARD_ID, PRD.BEGIN_DATE, PRD.END_DATE,
    		               PRD.BEGIN_DATE AS BEGIN_DATE_PROJECT, PRD.END_DATE AS END_DATE_PROJECT, PRD.PRODUCT_TYPE
                          FROM PROJECT PRD, PROJECT_TYPE PTP, PROJECT ISS, PROJECT_TYPE PTI, TASK_STATUS TS
                         WHERE PRD.PROJECT_TYPE = PTP.ID
                           AND TS.ID = PRD.PROJECT_STATUS
                           AND PTP.EXTERNALTYPE = :product
                           AND ISS.PARENT = PRD.ID
                           AND ISS.PROJECT_TYPE = PTI.ID
                           AND PTI.EXTERNALTYPE = :typeIssue
                           AND ISS.STATUS <> :deleted
                           AND PRD.ID <> :projectId
                           AND PRD.ORGANIZATION = :organizationId
                           AND PRD.STATUS = :active
                           AND PRD.PRODUCT_TYPE <> :managerType
                           AND (TS.CONTROL = :planning OR TS.CONTROL = :progress)
                       $filterString
                        $filterUserAcess
                   ) P
             LEFT JOIN PRODUCT PRO ON (PRO.PROJECT_ID = P.ID)
             LEFT JOIN OM_PRODUTO OMP ON OMP.NREXTERNAL_PRODUCT_ID = PRO.ID) P
         WHERE 1 = 1
           AND P.IDPERCRIARCHAMADO = 'S'
         $filter
        ORDER BY P.BEGIN_DATE_PROJECT DESC, P.END_DATE_PROJECT DESC
SQL;
        $params = array(
			'organizationId' => $organizationId,
			'projectId' => $projectId,
            'managerType' =>  Entities\ProjectType::TYPE_PRODUCT_CONTROL_MANAGEMENT,
            'progress' => Entities\TaskStatus::TASK_STATUS_CONTROL_IN_DEVELOPING,
            'planning' => Entities\TaskStatus::TASK_STATUS_CONTROL_NOT_INITIALIZED,
            'deleted' =>  Entities\Project::STATUS_LOGICAL_DELETED,
            'active' =>  Entities\Project::IS_ACTIVE,
            'typeIssue' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE,
            'product' => Entities\ProjectType::TYPE_PROJECT_CONTROL_PRODUCT
		);

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function findProductsExceptConcludedOrCanceled($organizationId, $filterString = '', $limit, $offset){
        $query = <<<SQL
        SELECT *
          FROM (SELECT *
                  FROM (SELECT PRD.ID, PRD.NAME, PRD.ID AS BOARD_ID, PRD.BEGIN_DATE, PRD.END_DATE,
		                       PRD.BEGIN_DATE AS BEGIN_DATE_PROJECT, PRD.END_DATE AS END_DATE_PROJECT, TS.CONTROL, PRD.PRODUCT_TYPE
                          FROM PROJECT PRD, PROJECT_TYPE PTP, PROJECT ISS, PROJECT_TYPE PTI, TASK_STATUS TS
                         WHERE PRD.PROJECT_TYPE = PTP.ID
                           AND TS.ID = PRD.PROJECT_STATUS
                           AND TS.CONTROL NOT IN(:concluded, :canceled)
                           AND PRD.PRODUCT_TYPE NOT IN (:managerType)
                           AND PTP.EXTERNALTYPE = :product
                           AND ISS.PARENT = PRD.ID
                           AND ISS.PROJECT_TYPE = PTI.ID
                           AND PTI.EXTERNALTYPE = :typeIssue
                           AND ISS.STATUS <> :deleted
                           AND PRD.ORGANIZATION = :organizationId
                           AND PRD.STATUS = :active )
                 WHERE 1 = 1
                $filterString )
SQL;
        $params = array(
			'organizationId' => $organizationId,
            'concluded' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'canceled' =>  Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'product' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_PRODUCT,
            'typeIssue' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE,
            'deleted' =>  Entities\Project::STATUS_LOGICAL_DELETED,
            'active' =>  Entities\Project::IS_ACTIVE,
            'managerType' =>  Entities\ProjectType::TYPE_PRODUCT_CONTROL_MANAGEMENT
		);
        $paginatedQuery = Utilities::getPaginatedQuery($query."P
			ORDER BY P.BEGIN_DATE_PROJECT DESC, P.END_DATE_PROJECT DESC", $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($paginatedQuery, $params);
	}

	public function findMyProducts($organizationId, $projectId, $userId, $filterString = '', $structures = ''){
        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);
		$query = <<<SQL
			SELECT *
			  FROM (SELECT P.ID, P.NAME, P.ID AS BOARD_ID ,B.ID AS PROJECT_ISSUE_ID, P.BEGIN_DATE, P.END_DATE, P.ALLOW_REPLICATION_LTS,
    			           TS.LABEL AS PROJECT_STATUS_LABEL, TS.ID AS PROJECT_STATUS_ID,RES.NAME AS CREATED_BY,P.NOTIFY_ISSUE_OPENING,P.PRODUCT_TYPE,
    			           TO_CHAR(P.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_AT
                      FROM PROJECT P
                     INNER JOIN	PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID
                     INNER JOIN	PROJECT B ON B.PARENT = P.ID AND B.STATUS <> :deleted
                     INNER JOIN	PROJECT_TYPE BT ON BT.ID = B.PROJECT_TYPE AND BT.EXTERNALTYPE = :typeIssue
                     INNER JOIN TASK_STATUS TS ON TS.ID = P.PROJECT_STATUS
                      LEFT JOIN RESOURCE_T RES ON(RES.ID = P.CREATED_BY )
                     WHERE P.ORGANIZATION = :organizationId
                       AND $stringProjectByManager
                       AND PT.EXTERNALTYPE = :product
                       AND P.STATUS = :active
                       AND P.ID <> :projectId
                       $structures) P
                     WHERE 1 = 1
             $filterString
                  ORDER BY P.BEGIN_DATE DESC, P.END_DATE DESC
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'projectId' => $projectId,
			'active' =>  Entities\Project::IS_ACTIVE,
			'product' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_PRODUCT,
			'deleted' =>  Entities\Project::STATUS_LOGICAL_DELETED,
			'typeIssue' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE
		);

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function findAllProducts($organizationId){
		$query = <<<SQL
			SELECT *
			  FROM (SELECT P.ID, P.NAME, P.ID AS BOARD_ID ,B.ID AS PROJECT_ISSUE_ID, P.BEGIN_DATE, P.END_DATE
                      FROM PROJECT P
                     INNER JOIN	PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID
                     INNER JOIN	PROJECT B ON B.PARENT = P.ID AND B.STATUS <> :deleted
                     INNER JOIN	PROJECT_TYPE BT ON BT.ID = B.PROJECT_TYPE AND BT.EXTERNALTYPE = :typeIssue
                     WHERE P.ORGANIZATION = :organizationId
                       AND PT.EXTERNALTYPE = :product
                       AND P.STATUS = :active
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'product' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_PRODUCT,
			'deleted' =>  Entities\Project::STATUS_LOGICAL_DELETED,
			'active' =>  Entities\Project::IS_ACTIVE,
			'typeIssue' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query.") P
			ORDER BY P.BEGIN_DATE DESC, P.END_DATE DESC", $params);
	}

    public function findSprints($organizationId, $products,$userId = 0)
    {
        $projectIds = array();
        foreach ($products as $product) {
            $projectIds[] = array('ID' => $product);
        }
        $projectsIdsString = $this->getChildrenProjectsId($projectIds);
        $stringProjectByManagerInString = Utilities::getStringSelectAllProjectManagerByUser($userId);
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PRT',true);
        $query = <<<SQL
			SELECT *
			  FROM (SELECT P.ID,
                    P.NAME ,PRT.LABEL ,
                    P.PARENT
                      FROM PROJECT P
                      JOIN PROJECT_TYPE PRT ON(P.PROJECT_TYPE = PRT.ID)
                     WHERE P.ORGANIZATION = :organizationId
                       AND (P.ID IN ($projectsIdsString) OR $stringProjectByManagerInString )
                       AND P.STATUS = :active
                       $filterTypeProject
SQL;
        $params = array(
            'organizationId' => $organizationId,
            'active' =>  Entities\Project::IS_ACTIVE
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query . ") P
			ORDER BY P.NAME ", $params);
    }

	/**
	 * @param $boardId
	 * @return array
     */
	public function selectBoard($boardId){
		$query = <<<SQL
			SELECT 	B.ID, B.NAME, BP.ID AS UPPER_PROJECT, BP.NAME AS UPPER_PROJECT_NAME, B.DESCRIPTION, R.ID AS MANAGER, R.NAME AS MANAGER_NAME,
					S.ID AS STRUCTURE, S.NAME AS STRUCTURE_NAME
			FROM 		PROJECT B
			LEFT JOIN 	PROJECT BP ON BP.ID = B.PARENT
			INNER JOIN 	RESOURCE_PROJECT RP ON RP.PROJECT = B.ID AND RP.TYPE = 'M'
			INNER JOIN 	RESOURCE_T R ON R.ID = RP.RESOURCE_C
			INNER JOIN 	STRUCTURE S ON S.ID = B.STRUCTURE
			WHERE   B.ID = :boardId
			AND		B.IS_ISSUE_PROJECT = 'Y'
			AND 	B.STATUS <> 'D'
SQL;

		$params = array(
			'boardId' => $boardId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function findProjectCategories($organizationId, $projectType = null, $filterLabelString = ''){
		$query = <<<SQL
		  SELECT * FROM (
              SELECT CLA.ID, CLA.LABEL
              FROM CLASSIFICATION CLA
              LEFT JOIN CLASSIFICATION_PROJECT_TYPE CPT ON (CPT.CLASSIFICATION = CLA.ID AND CPT.TYPE = 'N' AND  CPT.PROJECT_TYPE = :projectType)
    		  WHERE CLA.ORGANIZATION = :ORGANIZATION_ID
              AND (CPT.ID IS NULL )
    		  AND IS_PROJECT = 'Y')
		  WHERE 1=1
		  $filterLabelString
		  ORDER BY LABEL
SQL;
		$params = array(
			'ORGANIZATION_ID' => $organizationId,
            'projectType' => $projectType
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function findProjectTypes($organizationId, $filterName, $filterOperator, $filterValue){
        if($filterName) {
            $filterName = "UPPER(".$filterName.")";
            $filterValue = "UPPER('".$filterValue."')";
            $filterString = "AND $filterName $filterOperator $filterValue";
        } else {
            $filterString = "";
        }
		$query = <<<SQL

		   SELECT PRT.ID, PRT.LABEL,
		          LISTAGG(PRA.PROFILE_ID, ',') WITHIN GROUP (ORDER BY PRT.ID) AS PROFILE
			 FROM PROJECT_TYPE PRT
        LEFT JOIN PROJECT_APPROVAL PRA ON (PRA.PROJECT_TYPE_ID = PRT.ID)
			WHERE PRT.ORGANIZATION = :ORGANIZATION_ID
			$filterString
         GROUP BY PRT.ID, PRT.LABEL
SQL;
		$params = array(
			'ORGANIZATION_ID' => $organizationId,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    /**
     * @param $organizationId
     * @param bool $filterName
     * @param bool $filterOperator
     * @param bool $filterValue
     * @return array
     */
    public function findProjectStatusByOrganization($organizationId, $filterName = false, $filterOperator = false, $filterValue = false, $filter = ""){
        if($filterName && $filterName != "ID") {
            $filterString = "AND UPPER($filterName) $filterOperator UPPER('$filterValue')";
        } else {
            $filterString = "";
        }
        $query = "
        SELECT *
        FROM (
    		SELECT 		TS.ID, TS.ORGANIZATION, TS.CONTROL, TS.LABEL, TS.IS_ISSUE,
    					TS.IS_TASK, TS.IS_PROJECT, TS.TASK_CAN_START, TS.GENERATES_LOG
              FROM	    TASK_STATUS TS
              WHERE	    TS.ORGANIZATION = :organizationId
              AND       TS.IS_PROJECT = 'Y'
              $filterString
              ORDER BY 	ID ASC)
        WHERE 1=1
              $filter";

		$params = array(
			'organizationId' => $organizationId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function findProjectStatusInEditProject($filterString,$organizationId){

        $query = "
        SELECT 	TS.ID, TS.ORGANIZATION, TS.CONTROL, TS.LABEL, TS.IS_ISSUE,
					TS.IS_TASK, TS.IS_PROJECT, TS.TASK_CAN_START, TS.GENERATES_LOG
          FROM	    TASK_STATUS TS
          WHERE	    TS.ORGANIZATION = :organizationId
          AND       TS.IS_PROJECT = 'Y'
          $filterString
          ORDER BY 	ID ASC";
        $params = array(
            'organizationId' => $organizationId,
            'IN_PROGRESS' => Entities\TaskStatus::TASK_STATUS_CONTROL_PROGRESS,
        );
		$result = $this->getEntityManager()->getConnection()->fetchAll($query,$params);
		return $result;
	}

    /**
     * @param $organizationId
     * @param $concludedStatus
     * @return array
     */
    public function findProjectStatusByControlAndOrganization($concludedStatus, $organizationId){
        $query = "
		SELECT 		TS.ID, TS.ORGANIZATION, TS.CONTROL, TS.LABEL, TS.IS_ISSUE,
					TS.IS_TASK, TS.IS_PROJECT, TS.TASK_CAN_START, TS.GENERATES_LOG
          FROM	    TASK_STATUS TS
          WHERE	    TS.ORGANIZATION = :organizationId
          AND       TS.IS_PROJECT = 'Y'
          AND       TS.CONTROL = :concludedStatus
          ORDER BY 	ID ASC";
		$params = array(
			'concludedStatus' => $concludedStatus,
			'organizationId' => $organizationId
		);

		$result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		return $result;
	}

    /************************************************************
     * Gets different resources between project and task
     * @param $tasksIdsQuery
     * @param $projectId
     ***********************************************************
     * @internal param $tasksIds
     * @return array
     */
    public function getDifferentResourcesBetweenProjectAndTask($tasksIdsQuery, $projectId){
        $query = "
          SELECT * FROM (
          SELECT DISTINCT   RES.ID AS RESOURCE_ID, RES.NAME AS RESOURCE_NAME
          FROM              ALLOCATION ALLOC
          LEFT JOIN         RESOURCE_T RES    ON (RES.ID = ALLOC.RESOURCE_T)
          WHERE             ALLOC.TASK        IN $tasksIdsQuery
        ) WHERE NOT EXISTS(
          SELECT            RES.ID
          FROM              RESOURCE_PROJECT RESPROJ
          LEFT JOIN         RESOURCE_T RES    ON (RES.ID = RESPROJ.RESOURCE_C)
          WHERE             RESPROJ.PROJECT = :projectId
          AND               RES.ID = RESOURCE_ID
        )";

        $params = array(
            'tasksIdsQuery' => $tasksIdsQuery,
            'projectId' => $projectId
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    /**
     * @param $organizationId
     * @return array
     */
    public function findFirstProjectByOrganization($organizationId){
        $query = "SELECT ID
                    FROM PROJECT
                   WHERE ORGANIZATION = :organizationId
                     AND ROWNUM = 1";

        $params = array( 'organizationId' => $organizationId );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    /**
     * @param $parentProject
     * @return array
     */
    public function findTestProjectsByParent($parentProject){
        $query = "SELECT PR.ID, PR.NAME, PT.LABEL
                    FROM PROJECT PR
                   INNER JOIN PROJECT_TYPE PT ON PT.ID = PROJECT_TYPE
                   INNER JOIN TASK_STATUS TS ON TS.ID = PR.PROJECT_STATUS
                   WHERE PARENT = :PARENT
                     AND PR.STATUS = 'A'
                     AND PT.EXTERNALTYPE = :EXTERNALTYPE
                     AND TS.IS_PROJECT = 'Y'
                     AND TS.CONTROL NOT IN (:EXCLUDES_CONTROLS)";

        $params = array(
            'PARENT' => $parentProject,
            'EXTERNALTYPE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TEST,
            'EXCLUDES_CONTROLS' => array(Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
                                         Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED)
        );
        $paramTypes = array(
            'EXCLUDES_CONTROLS' => Connection::PARAM_STR_ARRAY,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
        return $result;
    }



    public function getProjectStatusAndType($projectId)
    {
        $query = <<<SQL
                  SELECT PR.ID, PR.PROJECT_TYPE, PR.PROJECT_STATUS
                    FROM PROJECT PR
                   WHERE PR.ID = :projectId
SQL;

        $params = array(
            'projectId' => $projectId,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($result) ? array() : array($result[0]['PROJECT_STATUS'],$result[0]['PROJECT_TYPE']);
    }

    public function findProjectByResource($userId, $currentDate, $dateInverseString)
    {
        $query = <<<SQL
            SELECT PRO.ID, PRO.NAME,
                   CASE WHEN (TO_CHAR(PRO.END_DATE, 'YYYYMMDD') <= :CURRENT_DATE_INVERSE_STRING
                              OR
                              TO_DATE(:CURRENT_DATE, 'DD/MM/YYYY') BETWEEN PRO.BEGIN_DATE AND PRO.END_DATE)
                        THEN 'Y'
                        ELSE 'N'
                    END AS IS_FILTER_DEFAULT
              FROM PROJECT PRO
              JOIN RESOURCE_PROJECT RSP ON RSP.PROJECT = PRO.ID
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
             WHERE RESOURCE_C = :USER_ID
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
SQL;
        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':CURRENT_DATE_INVERSE_STRING' => $dateInverseString,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS),
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findRecurringProjectsByResource($userId, $currentDate, $dateInverseString)
    {
        $query = <<<SQL
            SELECT PRO.ID, PRO.NAME
              FROM PROJECT PRO
              JOIN RESOURCE_PROJECT RSP ON RSP.PROJECT = PRO.ID
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
             WHERE RESOURCE_C = :USER_ID
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND PRT.EXTERNALTYPE = :TYPE_PROJECT_CONTROL_RECURRING_TASKS
SQL;
        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':CURRENT_DATE_INVERSE_STRING' => $dateInverseString,
            ':TYPE_PROJECT_CONTROL_RECURRING_TASKS' => Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProjectByResourceAndDay($userId, $currentDate)
    {
        $query = <<<SQL
            SELECT PRO.ID, PRO.NAME
              FROM PROJECT PRO
              JOIN RESOURCE_PROJECT RSP ON RSP.PROJECT = PRO.ID
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
             WHERE RESOURCE_C = :USER_ID
               AND TO_DATE(:CURRENT_DATE, 'DD/MM/YYYY') BETWEEN PRO.BEGIN_DATE AND PRO.END_DATE
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
SQL;
        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE),
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findProjectsWithAppropriateTasksAtDay($userId, $currentDate, $filter, $appropriationTime)
    {
        $query = <<<SQL
            SELECT ID, NAME, SUM(TOTAL_FAT) AS BILLED, SUM(TOTAL_NOT_FAT) AS NOT_BILLED, (SUM(TOTAL_FAT) + SUM(TOTAL_NOT_FAT)) AS TOTAL
             FROM (SELECT PRO.ID, 'PROJETO - ' || PRO.NAME AS NAME,
                          DECODE(APP.PROJECT_ID, NULL, APP.TOTAL, 0) AS TOTAL_FAT,
                          DECODE(APP.PROJECT_ID, NULL, 0, APP.TOTAL) AS TOTAL_NOT_FAT, 'A' AS DIFF
                     FROM APPROPRIATION APP
                     JOIN TASK    TAS ON TAS.ID = APP.TASK
                     JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT OR PRO.ID = APP.PROJECT_ID
                     JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE AND PRT.EXTERNALTYPE <> :TYPE_PROJECT_CONTROL_RECURRING_TASKS
                    WHERE APP.RESOURCE_T = :USER_ID
                      AND TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE
                      AND APP.TOTAL > 0
                    UNION ALL
                   SELECT PRO.ID, 'PROJETO - ' || PRO.NAME AS NAME, $appropriationTime AS TOTAL_FAT, 0 AS TOTAL_NOT_FAT, 'B' AS DIFF
                     FROM TASK TAS
                     JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                    WHERE 1 = 1
                      $filter )
         GROUP BY ID, NAME
SQL;

        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':TYPE_PROJECT_CONTROL_RECURRING_TASKS' => Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findUserTotalAppropriationAtDay($userId, $currentDate)
    {
        $query = <<<SQL
            SELECT NVL(APPROPRIATE_TOTAL, 0) AS APPROPRIATE_TOTAL
              FROM (SELECT SUM(APP.TOTAL)  AS APPROPRIATE_TOTAL
                      FROM APPROPRIATION APP
                     WHERE APP.RESOURCE_T = :USER_ID
                       AND TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE
                   )
SQL;

        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params)[0];
    }

    public function findAllProjects($filter, $limit, $offset, $filters)
    {
        $query = <<<SQL
            SELECT PRO.ID, PRO.NAME
              FROM PROJECT PRO
              JOIN PROJECT_TYPE PET ON PET.ID = PRO.PROJECT_TYPE
             WHERE 1 = 1
               AND PRO.STATUS = 'A'
             $filter
             $filters
             ORDER BY NAME
SQL;

        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function findOpenSprintByUser($resourceId, $addDayInDate)
    {
        $query = <<<SQL
            SELECT MOD.NAME AS MODEL, STR.NAME AS STRUCTURE, TO_CHAR(MOD.BEGIN_DATE, 'DD/MM/YYYY') AS BEGIN_DATE,
                   TO_CHAR(MOD.END_DATE, 'DD/MM/YYYY')  AS END_DATE, MOD.ID || STR.ID AS ID
              FROM PROJECT PRO
              JOIN PROJECT_TYPE PTP
                ON (PTP.ID = PRO.PROJECT_TYPE AND PTP.ALLOWENDTASK = 'Y')
              JOIN RESOURCE_PROJECT RSP
                ON RSP.PROJECT = PRO.ID
              JOIN RESOURCE_T RST
                ON RST.ID = RSP.RESOURCE_C
              JOIN TASK_STATUS TST
                ON TST.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_RELATIONSHIP PRR
                ON (PRR.PROJECT = PRO.ID AND PRR.TYPE = 'G')
              JOIN PROJECT MOD
                ON MOD.ID = PRR.PROJECT_RELATIONSHIP
              JOIN PROJECT_TYPE PTM
                ON (PTM.ID = MOD.PROJECT_TYPE AND PTM.USETIMEPLANNING = 'Y')
              JOIN STRUCTURE STR
                ON STR.ID = PRO.STRUCTURE
             WHERE RST.ID = :USER_ID
               AND TRUNC(SYSDATE) BETWEEN TRUNC(MOD.BEGIN_DATE) AND TRUNC((MOD.END_DATE + :ADD_DAY_IN_DATE))
               AND PRO.STATUS = 'A'
             GROUP BY STR.NAME, MOD.NAME, MOD.BEGIN_DATE, MOD.END_DATE, MOD.END_DATE, MOD.ID, STR.ID
             ORDER BY STR.NAME, MOD.BEGIN_DATE
SQL;
        $params = array(
            'USER_ID' => $resourceId,
            'ADD_DAY_IN_DATE' => $addDayInDate
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProjectsParentByProjectId($projectId)
    {
        $query = <<<SQL
            SELECT PROJECT_ID, STRUCTURE, PROJECT_MANAGER || ',' || STUCTURE_MANAGER AS MANAGER
              FROM (SELECT PR.ID AS PROJECT_ID, PR.STRUCTURE,
                           (SELECT rtrim(xmlagg(xmlelement(e,RP.RESOURCE_C ,',').extract('//text()')).GetClobVal(),',')
                              FROM RESOURCE_PROJECT RP
                             WHERE RP.PROJECT = PR.ID
                               AND RP.TYPE = 'M') AS PROJECT_MANAGER,
                           (SELECT rtrim(xmlagg(xmlelement(e,PE.RESOURCE_T,',').extract('//text()')).GetClobVal(),',')
                              FROM PERMISSION PE
                             WHERE PE.STRUCTURE = PR.STRUCTURE
                               AND PE.TYPE = 'M') AS STUCTURE_MANAGER
                      FROM PROJECT PR
                      JOIN TASK_STATUS PS ON PS.ID = PR.PROJECT_STATUS
                     WHERE PR.ID IN ( SELECT P.ID
                                        FROM PROJECT P
                                       WHERE P.STATUS = :IS_ACTIVE
                                  START WITH P.ID IN (:PROJECT_ID)
                            CONNECT BY PRIOR P.PARENT = P.ID)
                       AND PS.CONTROL NOT IN (:TASK_STATUS_CONTROL_CANCELED, :TASK_STATUS_CONTROL_CONCLUDED))

SQL;
        $params = array(
            'PROJECT_ID' => $projectId,
            'IS_ACTIVE'    => Entities\Project::IS_ACTIVE,
            'TASK_STATUS_CONTROL_CANCELED'    => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'TASK_STATUS_CONTROL_CONCLUDED'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findStructureParentByStructureIds($structureIdsString)
    {
        $query = <<<SQL
              SELECT DISTINCT P.RESOURCE_T
                FROM PERMISSION P
               WHERE P.STRUCTURE IN (SELECT DISTINCT S.ID
                                       FROM STRUCTURE S
                                      WHERE S.STATUS = :IS_ACTIVE
                                 START WITH S.ID IN ($structureIdsString)
                           CONNECT BY PRIOR S.PARENT = S.ID)
                 AND P.TYPE = :TYPE_MANAGER
SQL;
        $params = array(
            'IS_ACTIVE'     => Entities\Structure::IS_ACTIVE,
            'TYPE_MANAGER'  => Entities\Permission::TYPE_MANAGER
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findChildrenProjectOfDevOpsType($projectId)
    {
        $query = <<<SQL
              SELECT P.ID
                FROM PROJECT P
                JOIN PROJECT_TYPE PT ON PT.ID = P.PROJECT_TYPE
                JOIN TASK_STATUS PS ON PS.ID = P.PROJECT_STATUS
               WHERE P.STATUS = :IS_ACTIVE
                 AND PT.EXTERNALTYPE IN (:TYPE_ERROR, :TYPE_RELEASE, :TYPE_SUB_RELEASE)
                 AND PS.CONTROL NOT IN (:TYPE_CONCLUDED, :TYPE_CANCELED)
                 AND P.ID IN ( SELECT PRO.ID
                                 FROM PROJECT PRO
                                WHERE PRO.STATUS = :IS_ACTIVE
                           START WITH PRO.ID = :PROJECT_ID
                     CONNECT BY PRIOR PRO.ID = PRO.PARENT)
SQL;
        $params = array(
            'IS_ACTIVE'     => Entities\Project::IS_ACTIVE,
            'PROJECT_ID'  => $projectId,
            'TYPE_ERROR' => Entities\ProjectType::TYPE_PROJECT_CONTROL_ERROR,
            'TYPE_RELEASE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_RELEASE,
            'TYPE_SUB_RELEASE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_SUB_RELEASE,
            'TYPE_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'TYPE_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function validateParentProjectChoiceByProject($projectId, $parentProjectId)
    {
        $query = <<<SQL
            SELECT 1 AS CHILD
              FROM DUAL
             WHERE :PROJECT_ACTUAL IN ( SELECT ID
                                          FROM PROJECT
                                         WHERE ID <> :PROJECT_SELECTED
                                    START WITH ID = :PROJECT_SELECTED
                                    CONNECT BY ID = PRIOR PARENT)
SQL;
        $params = array(
            'PROJECT_SELECTED'  => $parentProjectId,
            'PROJECT_ACTUAL'    => $projectId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

}