
// FILE: src/config/config.js
(function () {

	var ZeedhiApp = angular.module("ZeedhiApp");
	
	window.templateUrls = window.templateUrls || [];
	var packageName = 'zh-editable-treelist';

	var hasPackageRegistered = window.templateUrls.some(function (templateUrl) {
		return templateUrl.package === packageName;
	});

	if (!hasPackageRegistered) {
		window.templateUrls.push({
			'package': packageName,
			'baseUrl': './bower_components/' + packageName + '/dist/templates/'
		});
	}
})();

Configuration(function (ContextRegister, eventEngine) {
	// register events 
    eventEngine.registerEvent('BeforeMoveItem', 'bforeMoveItem');
    eventEngine.registerEvent('AfterMoveItem', 'afterMoveItem');
});

// FILE: src/controllers/EditableTreelistController.js
var EditableTreelistController = (function(_super) {
	__extends(EditableTreelistController1, _super);

	function EditableTreelistController1($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query, CrosstabService, $filter, maskEngine, ZhUtil, $window, ApplicationContext, ZhFeatures, eventAggregator, DecodeValueFieldService) {
		var _ref = EditableTreelistController.__super__.constructor.apply(this, arguments);

		// BEGIN: Initialize
	 	var self = this;
		var CONTROLLER_KEY = 'EditableTreelistControllerInit';

		var methodsToLoad = {
			"afterLoadDataSource": function() {
				reloadTreeDataSource();
			}
		};

		Util.initializeComponent(this.widget, CONTROLLER_KEY, methodsToLoad, undefined, eventEngine);
		// END: Initialize

		// BEGIN: Tree
		var parentProperty = this.widget.parentProperty;
		var primaryProperty = this.widget.primaryProperty;
		var breadcrumbProperty = this.widget.breadcrumbProperty;
		var treeChildrenMap = {};

		self.itemHasSiblings = function(row, level) {
			if (row[parentProperty]) {
				var result = (row._level == level + 1);

				var rowIndex = self.sortableItems.indexOf(row);
				for (var i = rowIndex + 1; i < self.sortableItems.length; i++) {
					if (self.sortableItems[i]._level == level + 1) {
						result = true;
						break;
					} if (self.sortableItems[i]._level < level + 1) {
						break;
					}
				}

				return result;
			} else {
				return false;
			}
		};

		function _repeatFilter(item) {
			return self.repeatFilter(item) && (!self.widget.filterDataFunction || self.widget.filterDataFunction(item));
		}

		self.getSortableItems = function() {
			var items;
			if (self.widget.showAsTree) {
				items = (self.widget.plainTreeData || []).filter(_repeatFilter);
			} else {
				items = self.widget.groupedData;
			}

			if (!self.sortableItems || self.sortableItems.length != items.length) {
				self.sortableItems = items;
			}

			return self.sortableItems;
		};

		function _buildTreeData(data, childKey, parentKey, parentValue, level, parent) {

			var dataResult = data.filter(function(item) {
				return item[childKey] === parentValue || (parentValue === null && !item[childKey]);
			});

			dataResult.forEach(function(item) {
				item._parent = parent;
				// generate the breadcrumb for the item
				if (parent && breadcrumbProperty) {
					item._breadcrumb = (parent._breadcrumb ? parent._breadcrumb + ' > ' : '') + parent[breadcrumbProperty];
				}
				if (item[parentKey]) {
					item._children = _buildTreeData(data, childKey, parentKey, item[parentKey], level + 1, item);
				} else {
					item._children = [];
				}
				item._level = level;
			});

			if (self.widget.sortableFieldName) {
				dataResult.sort(function(a, b) {
					return a[self.widget.sortableFieldName] - b[self.widget.sortableFieldName];
				});
			}

			return dataResult;
		}

		function plain(data, result) {
			data.forEach(function(child) {
				var lastIndex = result.push(child);
				result[lastIndex - 1]._childCount = result[lastIndex - 1]._children.length;
				plain(child._children, result);

				treeChildrenMap[result[lastIndex - 1][primaryProperty]] = result[lastIndex - 1]._children;
				delete result[lastIndex - 1]._children;
			});
		}

		function reloadTreeDataSource() {
			if (self.widget.showAsTree) {
				self.widget.treeData = [];
				if (self.widget.dataSource && self.widget.dataSource.data) {
					treeChildrenMap = {};
					self.widget.treeData = _buildTreeData(self.widget.dataSource.data, parentProperty, primaryProperty, null, 1, null);
					self.widget.plainTreeData = [];
					plain(self.widget.treeData, self.widget.plainTreeData);
					if(self.widget.isOpened){
						self.widget.openAllGrid();
					}
				}

				self.sortableItems = (self.widget.plainTreeData || []).filter(_repeatFilter);
			}
		}

		function openOrCloseGrid(opened){
			self.widget.plainTreeData.forEach(function(row){
				if(row._childCount && row._childCount > 0){
					row._opened = opened;
				}
			});
		}

		if (self.widget.showAsTree) {
			self.widget.openAllGrid = function(){
				openOrCloseGrid(true);
			};

			self.widget.closeAllGrid = function(){
				openOrCloseGrid(false);
			};

			$scope.$watchCollection('widget.dataSource.data', reloadTreeDataSource);
		} else {
			$scope.$watchCollection('widget.groupedData', self.getSortableItems);
		}

		// self.sortableItems = self.getSortableItems();
		// var lastDragOverElement;
		// $scope.sortableOptions = {
		// 	placeholder: "editable-treelist-placeholder",
		// 	handle: ".editable-treelist-item-container",
		// 	revert: true,
		// 	beforeStop: function(e, ui) {
		// 		if ($scope.widget.beforeMoveItem) {
		// 			var event = $scope.widget.beforeMoveItem({
		// 				"item": ui.item.sortable.model,
		// 				"position": {
		// 					"from": ui.item.sortable.index,
		// 					"to": ui.item.index()
		// 				}
		// 			});
		// 			if (event.data === false) {
		// 				ui.item.sortable.cancel();
		// 			}
		// 		}
		// 	},
		// 	start: function(e, ui) {
		// 		$(e.target).parents('.zh-editable-treelist-container').removeClass('nonbounce'); // prevent scroll during dragging
		// 	},
		// 	sort: function(e, ui) {
		// 		if (self.widget.showAsTree) {
		// 			var elementDraggingOver = $(document.elementFromPoint(50, ui.offset.top - 5)).closest('li');
		// 			if (elementDraggingOver.length === 1 && ui.offset.top < elementDraggingOver.offset().top + elementDraggingOver.outerHeight() / 2) {
		// 				lastDragOverElement = elementDraggingOver;
		// 				elementDraggingOver.addClass('dragging-over');
		// 				$(e.target).removeClass('show-placeholder');
		// 			} else {
		// 				if (lastDragOverElement && lastDragOverElement.length === 1) {
		// 					lastDragOverElement.removeClass('dragging-over');
		// 					lastDragOverElement = undefined;
		// 				}
		// 				$(e.target).addClass('show-placeholder');
		// 			}

		// 		}
		// 	},
		// 	stop: function(e, ui) {
		// 		var itemModel = ui.item.sortable.model;
		// 		$(e.target).parents('.zh-editable-treelist-container').addClass('nonbounce'); // prevent scroll during dragging

		// 		if (self.widget.showAsTree) {
		// 			updateTreeData(ui.item);
		// 		}

		// 		var modifiedRows = getModifiedRows();
		// 		if (!ui.item.sortable.isCanceled()) {
		// 			if ($scope.widget.afterMoveItem) {
		// 				$scope.widget.afterMoveItem({
		// 					"item": ui.item.sortable.model,
		// 					"modifiedRows": modifiedRows
		// 				});
		// 			}
		// 		}
		// 	}
		// };

		// function updateTreeData(uiItem) {
		// 	item = $(uiItem[0]);

		// 	if (lastDragOverElement) {
		// 		uiItem.sortable.model[parentProperty] = lastDragOverElement.attr('data-primary-value');
		// 		uiItem.sortable.model._level = +lastDragOverElement.attr('data-row-level') + 1;
		// 	} else {
		// 		var prevParentId = item.prev().attr('data-parent-id');
		// 		var prevPrimaryValue = item.prev().attr('data-primary-value');
		// 		var prevLevel = +(item.prev().attr('data-row-level') || 0);
		// 		var nextLevel = +(item.next().attr('data-row-level') || 0);

		// 		if (nextLevel > prevLevel) {
		// 			uiItem.sortable.model[parentProperty] = prevPrimaryValue;
		// 			uiItem.sortable.model._level = prevLevel + 1;
		// 		} else {
		// 			uiItem.sortable.model[parentProperty] = prevParentId;
		// 			uiItem.sortable.model._level = prevLevel;
		// 		}
		// 		if (uiItem.sortable.model[parentProperty]) {
		// 			uiItem.sortable.model._parent = uiItem.sortable.sourceModel.filter(function(item) {
		// 				return item[primaryProperty] == uiItem.sortable.model[parentProperty];
		// 			})[0];
		// 		}
		// 	}
		// }

		function getModifiedRows() {
			var rows = [];
			self.sortableItems.forEach(function(row, idx) {
				if (row[self.widget.sortableFieldName] != idx) {
					row[self.widget.sortableFieldName] = idx;
					row._changed = true;
					rows.push(row);
				}
			});
			return rows;
		}

		self.toggleCollapsedItemsView = self.widget.toggleCollapsedItemsView = function() {
			self.widget.collapsedItemsView = !self.widget.collapsedItemsView;
			templateManager.updateTemplate();
			$rootScope.$broadcast('treelist.toggleCollapsedItemsView');
		};

		self.postRepeat = function() {
			templateManager.container.refresh();
			$rootScope.$broadcast('treelist.postRepeat');
		};

		var foundCount = 0;
		var foundSelect = 0;
		var doFind = Util.buildDebounceMethod(function(findStr) {
			var findableFields = self.widget.fields.filter(function(field) { return field.showOnList && field.aclIsVisible && field.isVisible && (self.widget.collapsedItemsView !== true || field.showOnCollapsedItemsView !== false); });
			foundCount = 0;
			foundSelect = 0;
			self.widget.plainTreeData.map(function(row) {
				row._found = !Util.isEmptyOrBlank(findStr) && findableFields.some(function(field) {
					return Boolean(~DecodeValueFieldService.model(field, row[field.name]).indexOfLatin(findStr));
				});
				if (row._found) {
					foundCount++;
				}
			});
			if (foundSelect === 0 && foundCount > 0) {
				foundSelect = 1;
				self.selectFoundRow(0);
			}
			$scope.$apply();
		}, 500);
		$scope.$watch('widget.findCriteria', doFind);

		self.getFoundCount = function() {
			return foundSelect + '/' + foundCount;
		};

		self.selectFoundRow = function(delta) {
			foundSelect += delta;
			if (foundSelect < 1) {
				foundSelect = 1;
			} else if (foundSelect > foundCount) {
				foundSelect = foundCount;
			} else {
				self.widget.currentRow = self.widget.plainTreeData.filter(function(row) { return row._found; })[foundSelect-1];

				openParentsAndScrollToCurrentRow(self.widget.currentRow);
			}
		};

		function openParentsAndScrollToCurrentRow(selectedRow, scroll) {
			if (selectedRow[parentProperty]) {
				var parentRow = self.widget.plainTreeData.filter(function(row) { return row[primaryProperty] == selectedRow[parentProperty]; })[0];
				parentRow._opened = true;
				openParentsAndScrollToCurrentRow(parentRow, false);
			}

			if (scroll !== false) {
				$timeout(function() {
					$scope.scrollToCurrentRow();
				});
			}
		}

		self.isParent = function(row) {
			return row[self.widget.isParentProperty] === true || row[self.widget.isParentProperty] == 'Y' || self.widget.dataSource.data.some(function(dataRow) {
				return dataRow[self.widget.parentProperty] == row[self.widget.primaryProperty];
			});
		};

		return _ref;
	}

	return EditableTreelistController1;
})(TreeController);

// FILE: src/directives/zhEditableTreelist.js
var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhEditableTreelist', function($rootScope, $timeout) {
	return {
		link: function($scope, element, attrs, controller) {
			var widget = $scope.widget;
			
			element.find('.zh-editable-treelist-container').on('scroll', function(e) {
				if (widget.onScroll) {
					widget.onScroll(null, {data: $(e.target).scrollTop()});	
				}
			});

			function hexToRgb(hex) {
			    // Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
			    var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
			    hex = hex.replace(shorthandRegex, function(m, r, g, b) {
			        return r + r + g + g + b + b;
			    });
			
			    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
			    return result ? {
			        r: parseInt(result[1], 16),
			        g: parseInt(result[2], 16),
			        b: parseInt(result[3], 16)
			    } : null;
			}

			$scope.getItemStyle = function(row) {
				var color = row[widget.itemColorProperty] || '#000';
				var colorRGB = hexToRgb(color);
				var style;
				
				if (widget.taskColor === false || widget.taskColor == 'none') {
					style = {
						'border-left-width': '1px',
						'border-left-color': '#ccc',
						'background-color': 'rgba(0,0,0,0.2)'
					};
				} else {
					if (widget.overrideTaskColor) {
						color = widget.overrideTaskColor;
						colorRGB = hexToRgb(color);
					}
					
					style = {
						'border-left-color': color,
						'background-color': 'rgba('+colorRGB.r+','+colorRGB.g+','+colorRGB.b+',0.2)'
					};
				}
				
				return style;
			};
			
			$scope.scrollToCurrentRow = function() {
				var currentItem = element.find('li.currentRow');
				if (currentItem.length > 0) {
					currentItem[0].scrollIntoView(true);
				}
			};
		}
	};
});

Configuration(function (ContextRegister, eventEngine) {
	// register the event finishDrawingEvent
	eventEngine.registerEvent('WidgetOnScroll', 'onScroll');
});